// TestInterface.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <windows.h>
#include <commctrl.h>
#include <time.h>
#include "lib\dev_std\usb_defs.h"
#include "LCARD_USB_DRIVER\WinCE\lcard_wince_usb_driver\ioctl.h"

/*
   FS520,  ARM   256 
  64  - 20 /
  128  - 40 /
  256  - 80 /
  512  - 150 /
  1024  - 252 /
  2048  - 405 /
  4096  -  - 530 /,  - 503 /
  8192  -  - 711 /,  - 613 /
*/

const DWORD SetTimeout=500;
const DWORD SizeofCycle = 8*1024;

double GetSpeed(DWORD Begin, DWORD End, int Bytes)
{
	double Totalms = End - Begin;
	
	if (Totalms)
	{
		return ((float)Bytes*1000.0)/(Totalms*1024.0);
	}
	else return 0;
}

bool ReadData(HANDLE device, BYTE * Data, DWORD DataLen, DWORD Timeout)
{
	
	if (!DeviceIoControl(device, IOCTL_CONTROL_SET_READ_BULK_TIMEOUT_MS, &Timeout, 
		sizeof(Timeout), NULL,0,NULL,NULL)) return false;
		
	DWORD BytesReturned=0;	
	if (ReadFile(device, Data, DataLen, &BytesReturned,0))
	{
		if (BytesReturned!=DataLen) return false;
		return true;
	}
	return false;
}

bool WriteData(HANDLE device, BYTE * Data, DWORD DataLen, DWORD Timeout)
{	
	/*
	if (!DeviceIoControl(device, IOCTL_CONTROL_SET_WRITE_BULK_TIMEOUT_MS, &Timeout, 
		sizeof(Timeout), NULL,0,NULL,NULL)) return false;
		*/
	DWORD BytesReturned=0;
	if (WriteFile(device, Data, DataLen, &BytesReturned,0))
	{
		if (BytesReturned!=DataLen) return false;
		return true;
	}
	return false;
}

void TestCycle()
{	
	HANDLE hFile=CreateFile(TEXT("DEV1:"), GENERIC_READ|GENERIC_WRITE, FILE_SHARE_READ|FILE_SHARE_WRITE,0,0,0,0);
	if (hFile!=INVALID_HANDLE_VALUE)
	{
		/*
		    Control  Bulk   INFINITE
		*/		
		BOOL Result=DeviceIoControl(hFile, IOCTL_CONTROL_SET_READ_BULK_TIMEOUT_MS, (BYTE *)&SetTimeout, sizeof(DWORD), NULL,0,NULL,NULL);

		_ControlRequest Request = {1,11,0,0};
		CHAR ModuleName[16];
		memset(ModuleName,0,sizeof(ModuleName));
		DWORD BytesReturned=0;
		Result=DeviceIoControl(hFile, IOCTL_CONTROL_PIPE_VENDOR, &Request,sizeof(Request), ModuleName, sizeof(ModuleName),&BytesReturned,0);
		if (Result)
		{
			_ControlRequest RequestCycle = {0,7,0,3};			
			DWORD SizeRead=0;
			BYTE TempDataWrite[SizeofCycle];
			BYTE TempDataRead[SizeofCycle];
			int ErrorCount=0;
			for (int i=0;i<SizeofCycle;i++)
			{
				TempDataWrite[i]=i;
			}
			Result=DeviceIoControl(hFile, IOCTL_CONTROL_PIPE_VENDOR, &RequestCycle,sizeof(RequestCycle), (BYTE *)&SizeofCycle, sizeof(SizeofCycle),&BytesReturned,0);
			if (Result)
			{				
				if (WriteData(hFile, TempDataWrite, SizeofCycle,SetTimeout))
				{
					memset(TempDataRead,0,sizeof(TempDataRead));
					if (ReadData(hFile, TempDataRead, SizeofCycle, SetTimeout))
					{					
						for (int i=0;i<SizeofCycle;i++)
						{
							if (TempDataRead[i]!=i)
							{
								ErrorCount++;
							}
						}
					}
					TempDataWrite[0]=0xff;					
					if (WriteData(hFile, TempDataWrite, SizeofCycle,SetTimeout))
					{
						Result=ReadData(hFile, (BYTE *)&SizeRead, sizeof(SizeRead), SetTimeout);
					}

					ErrorCount=0;
				}
			}
		}

		CloseHandle(hFile);
	}
}

double TestRead()
{	
	int ErrorCount=0;	
	int SuccsessCount=0;
	float ReadSpeed=0;
	DWORD BeginTime,EndTime;	
	DWORD SaveNumberOfBytes;
	HANDLE hFile=CreateFile(TEXT("DEV1:"), GENERIC_READ|GENERIC_WRITE, FILE_SHARE_READ|FILE_SHARE_WRITE,0,0,0,0);
	if (hFile!=INVALID_HANDLE_VALUE)
	{
		/*
		    Control  Bulk   INFINITE
		*/		
		BOOL Result=DeviceIoControl(hFile, IOCTL_CONTROL_SET_READ_BULK_TIMEOUT_MS, (BYTE *)&SetTimeout, sizeof(DWORD), NULL,0,NULL,NULL);

		_ControlRequest Request = {1,11,0,0};
		CHAR ModuleName[16];
		memset(ModuleName,0,sizeof(ModuleName));
		DWORD BytesReturned=0;
		Result=DeviceIoControl(hFile, IOCTL_CONTROL_PIPE_VENDOR, &Request,sizeof(Request), ModuleName, sizeof(ModuleName),&BytesReturned,0);
		if (Result)
		{
			_ControlRequest RequestCycle = {0,7,0,1};
			DWORD NumberBytes = SizeofCycle*512;
			DWORD NumDWORD	= NumberBytes>>2;
			SaveNumberOfBytes = NumberBytes;

			DWORD SizeRead=0;
			DWORD Counter=0;
			BYTE TempDataRead[SizeofCycle];			
			Result=DeviceIoControl(hFile, IOCTL_CONTROL_PIPE_VENDOR, &RequestCycle,sizeof(RequestCycle), (BYTE *)&NumDWORD, sizeof(NumDWORD),&BytesReturned,0);
			if (Result)
			{				
				BeginTime=GetTickCount();
				while (NumberBytes)
				{
					memset(TempDataRead,0,sizeof(TempDataRead));
					if(ReadData(hFile, TempDataRead, SizeofCycle, 500))
					{						
						for (int i=0;i<(SizeofCycle>>2);i++)
						{
							if (((DWORD *)TempDataRead)[i]!=Counter++)
							{
								ErrorCount++;
							}
							else SuccsessCount++;
						}
						NumberBytes-=SizeofCycle;
					}
					else 
					{
						ErrorCount++;					
					}
				}	
				EndTime =GetTickCount();
			}
		}

		CloseHandle(hFile);
	}

	if (ErrorCount) return -1;
	return GetSpeed(BeginTime, EndTime, SaveNumberOfBytes);
}

double TestWrite()
{		
	int ErrorCount=0;	
	int SuccsessCount=0;
	DWORD BeginTime, EndTime;	
	DWORD SaveNumberOfBytes;
	HANDLE hFile=CreateFile(TEXT("DEV1:"), GENERIC_READ|GENERIC_WRITE, FILE_SHARE_READ|FILE_SHARE_WRITE,0,0,0,0);
	if (hFile!=INVALID_HANDLE_VALUE)
	{
		/*
		    Control  Bulk   INFINITE
		*/		
		BOOL Result=DeviceIoControl(hFile, IOCTL_CONTROL_SET_READ_BULK_TIMEOUT_MS, (BYTE *)&SetTimeout, sizeof(DWORD), NULL,0,NULL,NULL);

		_ControlRequest Request = {1,11,0,0};
		CHAR ModuleName[16];
		memset(ModuleName,0,sizeof(ModuleName));
		DWORD BytesReturned=0;
		Result=DeviceIoControl(hFile, IOCTL_CONTROL_PIPE_VENDOR, &Request,sizeof(Request), ModuleName, sizeof(ModuleName),&BytesReturned,0);
		if (Result)
		{
			_ControlRequest RequestCycle = {0,7,0,2};
			DWORD NumberBytes = SizeofCycle*512;
			DWORD NumDWORD	= NumberBytes>>2;
			SaveNumberOfBytes = NumberBytes;

			DWORD SizeRead=0;
			DWORD Counter=0;
			BYTE TempDataWrite[SizeofCycle];			
			Result=DeviceIoControl(hFile, IOCTL_CONTROL_PIPE_VENDOR, &RequestCycle,sizeof(RequestCycle), (BYTE *)&NumDWORD, sizeof(NumDWORD),&BytesReturned,0);
			if (Result)
			{		
				BeginTime=GetTickCount();	

				while (NumberBytes)
				{
					for (int i=0;i<(SizeofCycle>>2);i++)
					{
						((DWORD *)TempDataWrite)[i]=Counter++;						
					}

					if(WriteData(hFile, TempDataWrite, SizeofCycle, 500))
					{						
						SuccsessCount++;						
						NumberBytes-=SizeofCycle;
					}
					else 
					{
						ErrorCount++;					
					}
				}

				EndTime=GetTickCount();	
				DWORD TempReadSize=0;
				if (ReadData(hFile,(BYTE *)&TempReadSize, sizeof(TempReadSize), 500))
				{
					if (TempReadSize!=0)
					{
						ErrorCount++;
					}
				} else ErrorCount++;
			}
		}

		CloseHandle(hFile);
	}

	if (ErrorCount) return -1;
	return GetSpeed(BeginTime, EndTime, SaveNumberOfBytes);
}


int _tmain(int argc, _TCHAR* argv[])
{
	TCHAR Message[256];	

/*
	DWORD BytesReturned=0;	
	_DEVICE_ID DevID={sizeof(_DEVICE_ID)};
	BYTE TempByte[256];
	CHAR PresetID[64];
	CHAR PlatformID[64];
	memset(PresetID,0,sizeof(PresetID));
	memset(PlatformID,0,sizeof(PlatformID));
	memset(&DevID,0,sizeof(DevID));
	memset(Message,0,sizeof(Message));
	memset(TempByte,0,sizeof(TempByte));	
	if (KernelIoControl(IOCTL_HAL_GET_DEVICEID, NULL,
                    0, TempByte, sizeof(TempByte), &BytesReturned))
	{
		memcpy(&DevID,TempByte,sizeof(DevID));
		int Counter=0;
		for (int i=DevID.dwPresetIDOffset;i<DevID.dwPresetIDOffset+DevID.dwPresetIDBytes;i++)
		{
			sprintf(PresetID+Counter,"%2x",i);
			Counter+=2;
		}
		Counter=0;
		for (int i=DevID.dwPlatformIDOffset;i<DevID.dwPlatformIDOffset+DevID.dwPlatformIDBytes;i++)
		{
			sprintf(PlatformID+Counter,"%2x",i);
			Counter+=2;
		}
		Counter=0;
	}
*/

	//TestData();
	double ReadSpeed=TestRead();
	double WriteSpeed=TestWrite();
	memset(Message,0,sizeof(Message));
	swprintf(Message,L"Read Speed %5.2f kb/s \n WriteSpeed %5.2f kb/s", ReadSpeed, WriteSpeed);
	MessageBox(NULL,Message,L"",0 );
	return 0;
}

