using System;
using System.IO;
using System.Threading;

namespace TestSpeed
{
    public class ErrorDataWork
    {
        public string ErrorFilePath = "";
        
        private int ErrorCount = 0;
        public int GetErrorCount
        {
            get
            {
                return ErrorCount;
            }
        }


        Mutex SyncMutex = new Mutex();
        public void ErrorProcess(System.Exception ex)
        {
            if (SyncMutex.WaitOne())
            {
                try
                {
                    ErrorCount++;                   

                    StreamWriter FileStream = new StreamWriter(ErrorFilePath, true, System.Text.Encoding.Unicode);

                    FileStream.WriteLine(": " + DateTime.Now.ToString("G"));

                    if (ex.Message != null) FileStream.WriteLine(": " + ex.Message);                    

                    if (ex.StackTrace != null) FileStream.WriteLine(": " + ex.StackTrace);

                    FileStream.WriteLine();

                    FileStream.Close();
                }
                catch { }
                finally
                {
                    SyncMutex.ReleaseMutex();
                }
            }
        }

        public void ErrorLog(string[] Errors)
        {
            if (SyncMutex.WaitOne())
            {
                try
                {
                    ErrorCount++;

                    StreamWriter FileStream = new StreamWriter(ErrorFilePath, true, System.Text.Encoding.Unicode);

                    FileStream.WriteLine(": " + DateTime.Now.ToString("G"));
                    for (int i = 0; i < Errors.Length; i++)
                    {
                        FileStream.WriteLine(Errors[i]);
                    }

                    FileStream.WriteLine();

                    FileStream.Close();
                }
                catch { }
                finally
                {
                    SyncMutex.ReleaseMutex();
                }
            }
        }

        public void ErrorLog(string Error)
        {
            if (SyncMutex.WaitOne())
            {
                try
                {
                    ErrorCount++;

                    StreamWriter FileStream = new StreamWriter(ErrorFilePath, true, System.Text.Encoding.Unicode);

                    FileStream.WriteLine(": " + DateTime.Now.ToString("G") + " | " + Error);
                    FileStream.Close();
                }
                catch { }
                finally
                {
                    SyncMutex.ReleaseMutex();
                }
            }
        }

        public string[] GetErrors()
        {
            string[] ReturnValue = null;
            if (SyncMutex.WaitOne())
            {
                try
                {
                    System.Collections.ArrayList StringArray = new System.Collections.ArrayList();

                    StreamReader FileStream = new StreamReader(ErrorFilePath, System.Text.Encoding.Unicode);

                    string TempString;
                    while ((TempString = FileStream.ReadLine()) != null)
                    {
                        StringArray.Add(TempString);
                    }

                    FileStream.Close();
                    ReturnValue = (string[])StringArray.ToArray(typeof(string));
                }
                catch { }
                finally
                {
                    SyncMutex.ReleaseMutex();
                }
            }

            return ReturnValue;
        }

        private static ErrorDataWork ThisInstance;
        public static ErrorDataWork Instance
        {
            get
            {
                if (ThisInstance == null) ThisInstance = new ErrorDataWork();
                return ThisInstance;
            }
        }

        private ErrorDataWork()
        {
            ErrorFilePath = System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase) + "\\Error.log";
            System.IO.File.Delete(ErrorFilePath);
        }
    }
}
