using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using CommonControls;
using CommonControls.WinCEApi;
using System.Collections;
using System.Threading;

namespace TestSpeed
{
    public partial class Form1 : Form
    {
        ErrorDataWork ErrorWork = ErrorDataWork.Instance;
        NativeIO ThisDevice = new NativeIO();
        const string DevName = "DEV";

        public Form1()
        {
            InitializeComponent();
            DisplayThreadDelegate = DisplayThread;
            comboBoxTestType.SelectedIndex = 0;
            FindDevice();
        }

        void FindDevice()
        {   
            comboBoxDevices.Items.Clear();
            NativeIO CurrentDevice = new NativeIO();
            for (int i = 0; i < 16; i++)
            {
                if (CurrentDevice.OpenDevice(DevName + i + ":"))
                {            
                    comboBoxDevices.Items.Add(CurrentDevice.GetDeviceName() + " Slot " + i);
                    CurrentDevice.CloseDevice();
                }
            }
            if (comboBoxDevices.Items.Count == 0)
            {
                comboBoxDevices.Text = "  ";
                comboBoxDevices.Refresh();
            }
            else
            {
                comboBoxDevices.SelectedIndex = 0;
            }
        }

        private void buttonFindDevices_Click(object sender, EventArgs e)
        {
            FindDevice();
        }

        private void comboBoxDevices_SelectedIndexChanged(object sender, EventArgs e)
        {
            ThisDevice.CloseDevice();
            int BeginPosition = comboBoxDevices.SelectedItem.ToString().LastIndexOf(" ")+1;
            if (!ThisDevice.OpenDevice(DevName + comboBoxDevices.SelectedItem.ToString().Substring(BeginPosition, comboBoxDevices.SelectedItem.ToString().Length - BeginPosition) + ":"))
            {
                comboBoxDevices.BackColor = Color.Red;
            }
            else
            {
                comboBoxDevices.BackColor = Color.LightGreen;
            }
        }

        int Number=0;
        bool ThisRun = false;
        bool DisplayRun = false;
        long BytesReaded = 0;
        long BytesWrited = 0;
        double ReadSpeed = 0;
        double WriteSpeed = 0;
        int ErrorCounter = 0;
        private void buttonTestStart_Click(object sender, EventArgs e)
        {
            if (buttonTestStart.Text == "")
            {
                if (comboBoxTestType.SelectedIndex == 0) return;

                ThisDevice.FlushUSBRead();
                ErrorCounter = 0;
                BytesWrited = 0;
                BytesReaded = 0;
                int Value = Convert.ToInt32(textBoxNumber.Text);
                Value = (Value / 256) * 256;
                textBoxNumber.Text = Value.ToString();
                NativeIO.Request CurrRequest = new NativeIO.Request();
                CurrRequest.bRequest = 7;
                CurrRequest.wIndex = (ushort)comboBoxTestType.SelectedIndex;                
                byte[] TempData = BitConverter.GetBytes((int)(Value>>2));
                if (comboBoxTestType.SelectedIndex == 3)
                { // 
                    TempData = BitConverter.GetBytes(256);
                }
               
                Thread ReadThread = new Thread(new ThreadStart(TestBulkRead));
                Thread WriteThread = new Thread(new ThreadStart(TestBulkWrite));      
                if (ThisDevice.ControlIO(CurrRequest, ref TempData, 500))
                {
                    ThisRun = true;
                    DisplayRun = true;
                    buttonTestStart.Text = "";
                    switch (comboBoxTestType.SelectedIndex)
                    {
                        case 1: //  
                            Number = Value;
                            ReadThread.Start();
                            break;
                        case 2:
                            Number = Value;
                            WriteThread.Start();
                            break;
                        case 3:
                            Number = -1;
                            ReadThread.Start();
                            WriteThread.Start();
                            break;
                        default:
                            break;
                    }

                    this.BeginInvoke(DisplayThreadDelegate);
                }                
            }
            else
            {
                ThisRun = false;
                buttonTestStart.Text = "";
            }
        }

        void TestBulkRead()
        {
            DateTime Begin = DateTime.Now;
            uint Counter = 0;
            ReadSpeed = 0;
            byte[] Data;
            while (Number != 0)
            {
                Data=ThisDevice.ReadData(256, 500);
                if (Data != null)
                {
                    for (int i = 0; i < Data.Length; i += 4)
                    {
                        if (BitConverter.ToInt32(Data, i) != Counter)
                        {
                            ErrorCounter++;
                            ErrorWork.ErrorLog("  ,  " + Counter + "  " + BitConverter.ToInt32(Data, i));
                        }
                        Counter++;
                    }
                   
                    if (Number > 0)
                    {
                        Number -= Data.Length;
                    }
                    BytesReaded += Data.Length;
                }

                if (!ThisRun)
                {
                    if (ReadSpeed == 0)
                    {
                        TimeSpan Difference = DateTime.Now-Begin;
                        ReadSpeed = BytesReaded * 1000.0/ (Difference.TotalMilliseconds * 1024);
                    }
                    if (Number >= 0) break;

                    if (WriteComplited)
                    {
                        Data = new byte[256];
                        Data[0] = 0xff;
                        while (BytesReaded != BytesWrited)
                        {
                            Data = ThisDevice.ReadData(256, 500);
                            if (Data != null)
                            {
                                BytesReaded += Data.Length;
                            }
                            else break;
                        }
                        ThisDevice.ReadData(256, 500);
                        if (ThisDevice.WriteData(Data, 500))
                        {
                            Data = ThisDevice.ReadData(4, 500);
                            if (Data != null)
                            {
                                if (BitConverter.ToInt32(Data, 0) != 256)
                                {
                                    ErrorWork.ErrorProcess(new Exception("  ,    256,  " + BitConverter.ToInt32(Data, 0)));
                                }
                            }
                        }
                        break;
                    }
                    else Thread.Sleep(100);
                }
            }
            ThisRun = false;
            DisplayRun = false;
        }

        bool WriteComplited = false;
        void TestBulkWrite()
        {            
            WriteComplited = false;
            WriteSpeed = 0;
            uint Counter = 0;            
            byte[] Data = new byte[256];
            byte[] TempByte;

            DateTime Begin = DateTime.Now;

            while (ThisRun && Number != 0)
            {
                for (int i = 0; i < Data.Length; i += 4)
                {
                    TempByte = BitConverter.GetBytes(Counter++);
                    TempByte.CopyTo(Data, i);
                };

                if (ThisDevice.WriteData(Data, 500))
                {
                    if (Number > 0)
                    {
                        Number -= Data.Length;
                    }
                    BytesWrited += Data.Length;
                }
                else
                {
                    ErrorCounter++;
                    ErrorWork.ErrorProcess(new Exception("  "));
                }             
            }
            if (WriteSpeed == 0)
            {
                TimeSpan Difference = DateTime.Now - Begin;
                WriteSpeed = BytesWrited * 1000.0 / (Difference.TotalMilliseconds * 1024);
            }

            if (Number >= 0)
            {
                ThisRun = false;
                byte[] NumError = ThisDevice.ReadData(4, 500);
                if (NumError != null)
                {
                    if (BitConverter.ToInt32(NumError,0) != 0)
                    {
                        ErrorWork.ErrorProcess(new Exception("    " + BitConverter.ToInt32(NumError,0)+" "));
                    }
                }
                DisplayRun = false;
            }
            WriteComplited = true;
        }

        delegate void DisplayThreadDelegateType ();
        DisplayThreadDelegateType DisplayThreadDelegate;

        void DisplayThread()
        {
            DateTime Begin = DateTime.Now;
            long BytesReadedLast = 0;
            long BytesWritedLast = 0;
            double ThisReadSpeed=0;
            double ThisWriteSpeed=0;
            TimeSpan Difference;
            while (DisplayRun)
            {
                for (int i = 0; i < 10; i++)
                {
                    Application.DoEvents();
                    Thread.Sleep(100);
                }                 
                
                Difference = DateTime.Now-Begin;
                
                if (ReadSpeed == 0)
                {
                    ThisReadSpeed = (BytesReaded - BytesReadedLast) * 1000 / (Difference.TotalMilliseconds * 1024);
                }
                else ThisReadSpeed = ReadSpeed;

                if (WriteSpeed == 0)
                {
                    ThisWriteSpeed = (BytesWrited - BytesWritedLast) * 1000 / (Difference.TotalMilliseconds * 1024);
                }
                else ThisWriteSpeed = WriteSpeed;

                Begin = DateTime.Now;
                BytesReadedLast = BytesReaded;
                BytesWritedLast = BytesWrited;

                textBoxSpeed.Text = " " + ThisReadSpeed.ToString("f1")+" /,"+
                    " " + ThisWriteSpeed.ToString("f1")+" /,"+
                    " "+(ThisReadSpeed+ThisWriteSpeed).ToString("f1")+" /";

                textBoxError.Text = ErrorCounter.ToString();
            }
            textBoxError.Text += " ";
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            ThisDevice.CloseDevice();
        }

        private void menuItem1_Click(object sender, EventArgs e)
        {
            this.Close();
        }
    }
}