using System;
using System.Collections.Generic;
using System.Text;
using CommonControls;
using CommonControls.WinCEApi;
using System.Runtime.InteropServices;
namespace TestSpeed
{
    class NativeIO
    {
        [DllImport("coredll.dll", SetLastError = true)]
        public static extern int DeviceIoControl(
            IntPtr hDevice,
            int dwIoControlCode,
            ref Request DeviceRequest,
            uint nInBufferSize,
            byte[] lpOutBuffer,
            uint nOutBufferSize,
            ref uint lpBytesReturned,
            IntPtr lpOverlapped);

        [StructLayout(LayoutKind.Explicit, Size = 8)]
        public struct Request
        {
            [FieldOffset(0)]
            public bool RequestDirectionIN; // 1 -   0 -  

            [FieldOffset(2)]
            public ushort bRequest;
            [FieldOffset(4)]
            public ushort wValue;
            [FieldOffset(6)]
            public ushort wIndex;
        }

        const int IOCTL_CONTROL_PIPE_VENDOR = 0x220012;

        const int IOCTL_CONTROL_SET_READ_BULK_TIMEOUT_MS = 0x220013;
        const int IOCTL_CONTROL_SET_WRITE_BULK_TIMEOUT_MS = 0x220014;
        const int IOCTL_CONTROL_SET_CONTROL_TIMEOUT_MS = 0x220015;
        const int IOCTL_CONTROL_GET_READ_BULK_TIMEOUT_MS = 0x220016;
        const int IOCTL_CONTROL_GET_WRITE_BULK_TIMEOUT_MS = 0x220017;
        const int IOCTL_CONTROL_GET_CONTROL_TIMEOUT_MS = 0x220018;

        public string GetDeviceName()
        {
            string Result = null;

            byte[] ModuleName = new byte[16];
            Request CurrentRequest = new Request();
            CurrentRequest.RequestDirectionIN = true;
            CurrentRequest.bRequest = 11;

            if (ControlIO(CurrentRequest, ref ModuleName, 1000))
            {
                Result = "";
                for (int i = 0; i < ModuleName.Length; i++)
                {
                    if (ModuleName[i] == 0) break;
                    Result += (char)ModuleName[i];
                }
            }
            else Result = "Undetected Device";

            return Result;
        }

        private IntPtr DevicePtr = coredll.InvalidHandleValue;

        public bool OpenDevice(string DeviceName)
        {
            DevicePtr = coredll.CreateFile(DeviceName,
                    coredll.GENERIC_READ | coredll.GENERIC_WRITE,
                    coredll.FILE_SHARE_READ | coredll.FILE_SHARE_WRITE,
                    IntPtr.Zero, 0, 0, IntPtr.Zero);

            if (DevicePtr != coredll.InvalidHandleValue)
            {
                return true;
            }
            else
            {
                return false;
            }
        }

        public void CloseDevice()
        {
            if (DevicePtr!=coredll.InvalidHandleValue) coredll.CloseHandle(DevicePtr);
        }

        public bool ControlIO(Request CurrentRequest, ref byte[] Data, int Timeout)
        {
            byte[] TimeoutByte = BitConverter.GetBytes(Timeout);
            uint BytesReturned = 0;
            if (coredll.DeviceIoControl(DevicePtr, IOCTL_CONTROL_SET_CONTROL_TIMEOUT_MS, TimeoutByte, (uint)TimeoutByte.Length, null, 0, ref BytesReturned, IntPtr.Zero) != 0)
            {             
                if (DeviceIoControl(DevicePtr, IOCTL_CONTROL_PIPE_VENDOR, ref CurrentRequest,
                      (uint)Marshal.SizeOf(CurrentRequest), Data, (uint)Data.Length, ref BytesReturned, IntPtr.Zero) != 0)
                {
                    return true;
                }
            }

            return false;            
        }

        public byte [] ReadData(uint Size, int Timeout)
        {
            byte[] TimeoutByte = BitConverter.GetBytes(Timeout);
            byte[] ZeroArray = new byte[1];
            uint BytesReturned = 0;

            if (Timeout < 500)
            {
                if (coredll.DeviceIoControl(DevicePtr, IOCTL_CONTROL_SET_READ_BULK_TIMEOUT_MS, TimeoutByte, (uint)TimeoutByte.Length, ZeroArray, 0, ref BytesReturned, IntPtr.Zero) == 0)
                    return null;
            }
            

            byte[] Data = new byte[Size];

            if (coredll.ReadFile(DevicePtr, Data, Size, out BytesReturned, IntPtr.Zero))
            {
                if (BytesReturned != Data.Length) return null;
                return Data;
            }
            else return null;
        }

        public bool WriteData(byte[] Data, int Timeout)
        {
            byte[] TimeoutByte = BitConverter.GetBytes(Timeout);            
            uint BytesReturned = 0;
            if (Timeout < 500)
            {
                if (coredll.DeviceIoControl(DevicePtr, IOCTL_CONTROL_SET_WRITE_BULK_TIMEOUT_MS, TimeoutByte, (uint)TimeoutByte.Length, null, 0, ref BytesReturned, IntPtr.Zero) == 0)
                    return false;
            }

            if (coredll.WriteFile(DevicePtr, Data, (uint)Data.Length, out BytesReturned, IntPtr.Zero))
            {
                if (BytesReturned != Data.Length) return false;
                return true;
            }
            else return false;
        }

        public void FlushUSBRead()
        {
            if (ReadData(64, 100) != null)
            {
                while (ReadData(64, 100) != null) ;
            }
        }

        public NativeIO()
        {
        }
    }
}
