#include "stdafx.h"
#include "driver.h"

//===============================================================================
//
//===============================================================================
BOOL USBInstallDriver(LPCWSTR szDriverLibFile)
{
	AddLog("Begin Install Driver");
	// registry keys that need to be setup:
	const WCHAR wsUsbDeviceID[] = CLASS_NAME_SZ;
	// register a unique string to identify this USB client driver:
	// creates HKLM\Drivers\USB\ClientDrivers\FPS_Class key...
	if ( !RegisterClientDriverID( wsUsbDeviceID ) )
		return FALSE;
	// registry key that need to be setup:
	WCHAR wsSubClassRegKey[] =CLIENT_REGKEY_SZ;
	// registry values that need to be written:
	REG_VALUE_DESCR usbKeyValues[] =
	{
		(TEXT("Dll")), REG_SZ, 0, (PBYTE)(szDriverLibFile),
		(TEXT("Prefix")), REG_SZ, 0, (PBYTE)(DEVICE_PREFIX),
		NULL, 0, 0, NULL
	};
	// Add information needed to load client driver...
	GetSetKeyValues( wsSubClassRegKey,
		&usbKeyValues[0],
		SET,
		TRUE );
	// Settings that cause this driver to be loaded in the future:
	USB_DRIVER_SETTINGS usbDriverSettings =
	{
		sizeof(USB_DRIVER_SETTINGS), // dwCount?
		USB_LCARD_DEVICE_VENDOR_ID, // dwVendorId?
		USB_NO_INFO, // dwProductId?
		USB_NO_INFO, // dwReleaseNumber?
		USB_NO_INFO, // dwDeviceClass?
		USB_NO_INFO, // dwDeviceSubClass?
		USB_NO_INFO, // dwDeviceProtocol?
		USB_NO_INFO, // dwInterfaceClass?
		USB_NO_INFO, // dwInterfaceSubClass?
		USB_NO_INFO // dwInterfaceProtocol?
	};
	// Establish the USB driver load settings for this driver. Once
	// registered, this device driver will load automatically without
	// prompting the user for the driver name. The
	// RegisterClientSettings function creates a registry key
	// HKLM\Drivers\USB\LoadClients\2303\Default\Default\FPS_Class
	// and adds an entry for the name of this driver DLL...

	WORD SupportedDevicesPID[] = {SupportedDevicesPIDs};
	int DeviceCounter= sizeof(SupportedDevicesPID)/sizeof(DWORD);

	while (DeviceCounter--)
	{
		usbDriverSettings.dwProductId=SupportedDevicesPID[DeviceCounter];

		if (RegisterClientSettings(szDriverLibFile, wsUsbDeviceID, NULL,&usbDriverSettings ))
		{
			AddLog("Driver Installed Succesfull");
			return TRUE;
		}
	}
	AddLog("Driver Installed UnSuccesfull");
	return FALSE;
}
//===============================================================================
//
//===============================================================================
/*++
===============================================================================
Function USBDeviceAttach
This function is a universal serial bus (USB) device attach routine for the
AES Finger Print Sensor.
Parameters
hDevice [in] Handle to a USB device.
lpUsbFuncs [in] Pointer to a USB device interface function
table.
lpUsbInterface [in] Pointer to the USB_INTERFACE structure that
contains interface information if a client is
loaded as an interface driver. If a client is
not loaded for a specific interface, this
parameter is NULL, and the client can get
interface information through the
FindInterface function.
szUniqueDriverId [in] Client driver identifier string.
fAcceptControl [out] Indicator if the driver accepts control of
the device or if the driver should continue
to try to load client drivers. Set to TRUE
if the driver accepts control of the device,
or FALSE if USB driver should continue to
try to load client drivers.
lpUsbDriverSettings [in] Pointer to the USB_DRIVER_SETTINGS structure
that indicates how the driver is loaded.
dwUnused [in] Reserved for use with future versions of USB
driver.
Return Value
Errors
===============================================================================
*/
BOOL 
USBDeviceAttach(USB_HANDLE hDevice,
				LPCUSB_FUNCS lpUsbFuncs,
				LPCUSB_INTERFACE lpUsbInterface,
				LPCWSTR szUniqueDriverId,
				LPBOOL fAcceptControl,
				LPCUSB_DRIVER_SETTINGS lpUsbDriverSettings,
				DWORD dwUnused)
{
	AddLog("-----------------------------");
	AddLog("Begin Attach Device");
	BOOL bRc = TRUE;
	PUSBFPS_CONTEXT pUsbFps = NULL;
	WCHAR wsSubClassRegKey[] = CLIENT_REGKEY_SZ;	
	
	DEBUGMSG(ZONE_USB_INIT,(TEXT(">USBDeviceAttach(0x%x, %s)\n"), hDevice, szUniqueDriverId));
	// Initially, say that we don't control this USB peripheral...
	*fAcceptControl = FALSE;
	do
	{
		// we think we found a device & interface we control,
		// so create our device context
		pUsbFps = (PUSBFPS_CONTEXT) LocalAlloc(LPTR, sizeof(USBFPS_CONTEXT));
		if ( !pUsbFps )
		{
			AddLog("Attach Alloc Error");
			// not enough memory, don't bother to install the driver...
			DEBUGMSG(ZONE_ERR,(TEXT("LocalAlloc error:%d\n"), GetLastError()));
			bRc = FALSE;
			break;
		}
		// add our own unique signature to the context so that we can validate
		// the context when it is passed to us in other functions...
		pUsbFps->ulSignature = USB_LCARD_ARM_SIG;
		InitializeCriticalSection(&pUsbFps->Control.Lock);
		InitializeCriticalSection(&pUsbFps->BulkIn.Lock);
		InitializeCriticalSection(&pUsbFps->BulkOut.Lock);
		// keep track of the parameters passed into this function...
		pUsbFps->hUsbDevice = hDevice;
		pUsbFps->lpUsbFuncs = lpUsbFuncs;
		// initially, this driver is not opened:
		pUsbFps->Flags.Open = FALSE;
		pUsbFps->Flags.UnloadPending = FALSE;
		// timeouts
		pUsbFps->BulkIn.Timeout=INFINITE;
		pUsbFps->BulkOut.Timeout=INFINITE;
		pUsbFps->Control.Timeout=INFINITE;
		// create an event that may be used to gracefully exit the driver
		// if the cord is yanked while we're talking to the device...
		pUsbFps->hCloseEvent = CreateEvent(NULL, AUTO_RESET_EVENT, FALSE, NULL);
		if ( !pUsbFps->hCloseEvent )
		{
			AddLog("Create Event Attach Error");
			DEBUGMSG(ZONE_ERR, (TEXT("CreateEvent error:%d\n"), GetLastError()));
			bRc = FALSE;
			break;
		}
		// set up the USB interface pipes...
		bRc = SetUsbInterface( pUsbFps, szUniqueDriverId);
		if ( !bRc )
		{
			AddLog("Error Setup USB Interface Pipes");
			DEBUGMSG( ZONE_ERR, (TEXT("SetUsbInterface failed!\n")));
			break;
		}
		// start the stream driver interface:
		//
		// ActivateDevice() loads a device driver and reads the registry key
		// specified in wsSubClassRegKey to get the DLL name, device prefix,
		// etc. Next, it adds the device to the active device list in the
		// HKEY_LOCAL_MACHINE\Drivers\Active registry key, and stores the
		// relevant values there. If you do not specify a device index in
		// the registry subkey specified in wsSubClassRegKey, then
		// ActivateDevice assigns a free index. Then ActivateDevice()
		// loads the device driver DLL in the process space of the Device
		// Manager. Then the Device Manager sends a device notification
		// message for the new device, i.e. this causes AES_Init() to run...
		pUsbFps->hStreamDevice = ActivateDevice( wsSubClassRegKey,(DWORD) pUsbFps );
		// the return value can be used to deactivate the device when needed...
		if ( pUsbFps->hStreamDevice )
		{
			// registers a callback for device notifications,
			// i.e. USB_CLOSE_DEVICE...
			bRc = lpUsbFuncs->lpRegisterNotificationRoutine(hDevice,DeviceNotify, pUsbFps );
			if ( !bRc )
			{
				AddLog("RegisterNotificationRoutine error");
				DEBUGMSG(ZONE_ERR,(TEXT("RegisterNotificationRoutine error:%d\n"),GetLastError()));
				break;
			}
		}
		else
		{
			// the streams interface failed to init, no use starting...
			DEBUGMSG(ZONE_ERR, (TEXT("ActivateDevice error:%d\n"), GetLastError()));
			bRc = FALSE;
			break;
		}
	} while(0);

	// were errors detected?
	if ( !bRc )
	{
		AddLog("Attach Device Failed");
		// If not our device, or error, then clean up
		RemoveDeviceContext( pUsbFps );
	}
	else
	{
		AddLog("Attach Device succesfull");
		// we are ready to control this device!
		*fAcceptControl = TRUE;		
	}
	DEBUGMSG(ZONE_USB_INIT, (TEXT("<USBDeviceAttach:%d\n"), *fAcceptControl));
	return bRc;

}

/*++
===============================================================================
Function RemoveDeviceContext
This function deallocates memory, closes handles, and deletes critical
sections associated with a specific device context of a Finger Print Sensor.
Parameters
pUsbFps [in] Pointer to the device context information
Return Value
void
===============================================================================
*/
VOID RemoveDeviceContext(PUSBFPS_CONTEXT pUsbFps)
{
	DEBUGMSG(ZONE_INIT,(TEXT(">RemoveDeviceContext(%p)\n"), pUsbFps));
	if ( VALID_CONTEXT( pUsbFps ) )
	{
		if (pUsbFps->Flags.Open )
		{
			AddLog("RemoveDeviceContext on open device!");
			DEBUGMSG(ZONE_ERR,(TEXT("RemoveDeviceContext on open device!\n")));
			return;
		}
		// close each of the pipes that are open...
		if (pUsbFps->BulkIn.hPipe && pUsbFps->lpUsbFuncs)
		{			
			pUsbFps->lpUsbFuncs->lpAbortPipeTransfers(pUsbFps->BulkIn.hPipe,0);
			pUsbFps->lpUsbFuncs->lpClosePipe(pUsbFps->BulkIn.hPipe);
		}
		// close each of the events associated to the pipes...
		if (pUsbFps->BulkIn.hEvent)
		{
			CloseHandle(pUsbFps->BulkIn.hEvent);
		}
		// close each of the pipes that are open...
		if (pUsbFps->BulkOut.hPipe && pUsbFps->lpUsbFuncs)
		{
			pUsbFps->lpUsbFuncs->lpAbortPipeTransfers(pUsbFps->BulkOut.hPipe,0);
			pUsbFps->lpUsbFuncs->lpClosePipe(pUsbFps->BulkOut.hPipe);
		}
		// close each of the events associated to the pipes...
		if (pUsbFps->BulkOut.hEvent)
		{
			CloseHandle(pUsbFps->BulkOut.hEvent);
		}		
		
		if (pUsbFps->ControlPipeEvent)
		{
			CloseHandle(pUsbFps->ControlPipeEvent);
		}

		// close the event for closing down the driver...
		if (pUsbFps->hCloseEvent)
		{
			CloseHandle(pUsbFps->hCloseEvent);
		}
		// free up memory...
		if (&pUsbFps->Control.Lock)
		{
			DeleteCriticalSection( &pUsbFps->Control.Lock );
		}
		if (&pUsbFps->BulkIn.Lock)
		{
			DeleteCriticalSection( &pUsbFps->BulkIn.Lock );
		}
		if (&pUsbFps->BulkOut.Lock)
		{
			DeleteCriticalSection( &pUsbFps->BulkOut.Lock );
		}
		// free the memory used by this driver...
		LocalFree(pUsbFps);
	}
	else
	{
		DEBUGMSG(ZONE_ERR,(TEXT("Invalid Parameter\n")));
	}
	DEBUGMSG(ZONE_INIT,(TEXT("<RemoveDeviceContext\n")));
}

/*++
===============================================================================
Function SetUsbInterface
This function associates pipe handles with the current device. There should
be two "EndPoints" for this device, the Bulk input, and the Bulk output. This
routine scans the known endpoints of this device and opens pipes to talk to
each one.
Parameters
pUsbFps [in] Pointer to the device context information
Return Value
TRUE if both of the pipes were opened, FALSE otherwise.
===============================================================================
*/
BOOL SetUsbInterface(PUSBFPS_CONTEXT pUsbFps,  LPCWSTR UniqueDriverId)
{
	LPCUSB_INTERFACE lpUsbInterface;
	LPCUSB_ENDPOINT pEndpoint;
	LPCUSB_DEVICE pDevice;
	DWORD dwIndex;
	BOOL bRc;
	DEBUGMSG( ZONE_USB_INIT, (TEXT(">SetUsbInterface\n")));
	if (!VALID_CONTEXT(pUsbFps))
	{
		AddLog("SetUsbInterface Context Validation Failed");
		DEBUGMSG( ZONE_ERR, (TEXT("Invalid parameter\n")));
		return FALSE;
	}
	// gets a pointer to a device information structure, LPCUSB_DEVICE...
	pDevice = pUsbFps->lpUsbFuncs->lpGetDeviceInfo(pUsbFps->hUsbDevice);
	if ( !pDevice )
	{
		AddLog("Error No Device Information available!");
		DEBUGMSG( ZONE_ERR, (TEXT("No Device Information available!\n")));
		return FALSE;
	}
	// search for a specific interface on a USB device. Get a
	// LPCUSB_INTERFACE back if the interface was found...
	lpUsbInterface = pUsbFps->lpUsbFuncs->lpFindInterface(pDevice, 0, 0);
	// did we get the pointer to the interface?
	if ( !lpUsbInterface )
	{
		// no interface found, can't associate pipes with one, so...
		AddLog("Error Invalid parameter lpUsbInterface");
		DEBUGMSG( ZONE_ERR, (TEXT("Invalid parameter lpUsbInterface\n")));
		return FALSE;
	}
	// parse through the endpoints and try to find input and output pipes...
	for ( dwIndex = 0;dwIndex < lpUsbInterface->Descriptor.bNumEndpoints;dwIndex++)
	{
		AddLog("Begin Look Endpoint");
		// point to the next endpoint for this device in its list...
		pEndpoint = lpUsbInterface->lpEndpoints + dwIndex;

		// Finger Print Sensor Class supports 1 Bulk OUT and 1 Bulk IN,
		// detect each one by their direction bit...
		// is this the output endpoint?
		if ( USB_ENDPOINT_DIRECTION_OUT(pEndpoint->Descriptor.bEndpointAddress) )
		{
			AddLog("Found Out Endpoint");
			// do we already have a handle to the output pipe?
			// is this is a bulk output connection point?
			if ((NULL == pUsbFps->BulkOut.hPipe) &&((pEndpoint->Descriptor.bmAttributes & USB_ENDPOINT_TYPE_MASK) ==USB_ENDPOINT_TYPE_BULK))
			{
				// open the Bulk OUT pipe for the USB device...
				pUsbFps->BulkOut.hPipe =
					pUsbFps->lpUsbFuncs->lpOpenPipe(pUsbFps->hUsbDevice,&pEndpoint->Descriptor);
				// were we successful in obtaining the handle to the pipe?
				if ( !pUsbFps->BulkOut.hPipe )
				{
					AddLog("OpenPipe BulkOut error ");
					DEBUGMSG(ZONE_ERR,(TEXT("OpenPipe error:%d\n"), GetLastError()));
					break;
				}
				// setup an endpoint specific event...
				pUsbFps->BulkOut.hEvent = CreateEvent(NULL,	MANUAL_RESET_EVENT,	FALSE,NULL);
				if ( !pUsbFps->BulkOut.hEvent )
				{
					AddLog("OpenPipe BulkOut CreateEvent error ");
					DEBUGMSG(ZONE_ERR,(TEXT("CreateEvent error:%d\n"), GetLastError()));
					return FALSE;
				}
				// save the endpoint address for future reference...
				pUsbFps->BulkOut.bIndex = pEndpoint->Descriptor.bEndpointAddress;
				pUsbFps->BulkOut.maxPacketSize=pEndpoint->Descriptor.wMaxPacketSize;
				AddLogMessage1("BulkOut wMaxPacketSize %d", pEndpoint->Descriptor.wMaxPacketSize);
			}
		}
		// is this the input endpoint?
		else if (USB_ENDPOINT_DIRECTION_IN( pEndpoint->Descriptor.bEndpointAddress ) )
		{
			AddLog("Found In Endpoint");
			// do we already have a handle to the input pipe?
			// is this is a bulk input connection point?
			if ((NULL == pUsbFps->BulkIn.hPipe) &&((pEndpoint->Descriptor.bmAttributes & USB_ENDPOINT_TYPE_MASK) == USB_ENDPOINT_TYPE_BULK))
			{
				// open the Bulk IN pipe for the USB device...
				pUsbFps->BulkIn.hPipe =	pUsbFps->lpUsbFuncs->lpOpenPipe( pUsbFps->hUsbDevice,&pEndpoint->Descriptor );
				if ( !pUsbFps->	BulkIn.hPipe )
				{
					AddLog("OpenPipe BulkIn error ");
					DEBUGMSG(ZONE_ERR,(TEXT("OpenPipe error: %d\n"), GetLastError()));
					break;
				}
				// setup an endpoint specific event...
				pUsbFps->BulkIn.hEvent = CreateEvent(NULL,MANUAL_RESET_EVENT,FALSE,NULL);
				if ( !pUsbFps->BulkIn.hEvent )
				{
					AddLog("OpenPipe BulkIn CreateEvent error ");
					DEBUGMSG(ZONE_ERR,(TEXT("CreateEvent error:%d\n"), GetLastError()));
					return FALSE;
				}
				// save the endpoint address for future reference...
				pUsbFps->BulkIn.bIndex = pEndpoint->Descriptor.bEndpointAddress;	
				pUsbFps->BulkIn.maxPacketSize=pEndpoint->Descriptor.wMaxPacketSize;
				AddLogMessage1("BulkIn wMaxPacketSize %d", pEndpoint->Descriptor.wMaxPacketSize);
			}
		}
		else
		{
			AddLog("Unsupported Endpoint");
			DEBUGMSG(ZONE_WARN,(TEXT("Unsupported Endpoint:0x%x\n"),pEndpoint->Descriptor.bEndpointAddress));
		}

	}
	// did we find our endpoints? Did we get handles to our pipes?
	// if we failed to find all of our endpoints then cleanup will occur later
	if(pUsbFps->BulkOut.hPipe && pUsbFps->BulkIn.hPipe) 
	{
		pUsbFps->ControlPipeEvent= CreateEvent(NULL,MANUAL_RESET_EVENT,FALSE,NULL);
		if ( !pUsbFps->ControlPipeEvent )
		{
			AddLog("Controlpipe CreateEvent error ");
			DEBUGMSG(ZONE_ERR,(TEXT("CreateEvent error:%d\n"), GetLastError()));
			return FALSE;
		}
		bRc = TRUE;
	}
	else
	{
		bRc = FALSE;
		AddLog("Error In one USB Pipe");
	}
	
	DEBUGMSG( ZONE_USB_INIT, (TEXT("<SetUsbInterface:%d\n"), bRc));
	return (bRc);
}
/*++
===============================================================================
Function DeviceNotify
This function is registered with the system using the
RegisterNotificationRoutine(). It provides the system with a way of notifying
the device driver when the USB Device is being removed from the
system so that it can clean up / uninstall the driver.
Parameters
lpvNotifyParameter [in] This is the value that was passed to the
RegisterNotificationRoutine() as the
lpvNotifyParameter when this device was installed.
dwCode [in] USB_CLOSE_DEVICE is currently the only supported
notification in WinCE.
*lpdwInfoX [in] Not used.
Return Value
Returns TRUE if the USB_CLOSE_DEVICE notification was processed, FALSE
otherwise.
===============================================================================
*/
BOOL WINAPI DeviceNotify(LPVOID lpvNotifyParameter,
						 DWORD dwCode,
						 LPDWORD *lpdwInfo1,
						 LPDWORD *lpdwInfo2,
						 LPDWORD *lpdwInfo3,
						 LPDWORD *lpdwInfo4)
{

	AddLog("DeviceNotify");

	PUSBFPS_CONTEXT pUsbFps = (PUSBFPS_CONTEXT) lpvNotifyParameter;
	DWORD dwWaitReturn;
	BOOL bRc;
	UNREFERENCED_PARAMETER(lpdwInfo1);
	UNREFERENCED_PARAMETER(lpdwInfo2);
	UNREFERENCED_PARAMETER(lpdwInfo3);
	UNREFERENCED_PARAMETER(lpdwInfo4);
	DEBUGMSG( ZONE_USB_INIT, (TEXT(">DeviceNotify\n")));
	if ( !VALID_CONTEXT( pUsbFps ) )
	{
		AddLog("DeviceNotify Invalid Context Error");
		DEBUGMSG( ZONE_ERR, (TEXT("Invalid Context!\n")));
		return FALSE;
	}
	switch( dwCode )
	{
	case USB_CLOSE_DEVICE:
		DEBUGMSG( ZONE_USB_INIT, (TEXT("USB_CLOSE_DEVICE\n")));
		if ( pUsbFps->Flags.Open )
		{
			// When an app has an open handle to this device and then
			// the user unplugs the USB cable the driver will unload
			// here. However, we don't want to unload the driver until
			// the device has been closed and it's safe to clean up.
			// So... set the driver state to reject all I/O and wait
			// for the driver to signal that it has closed...
			EnterCriticalSection( &pUsbFps->Control.Lock );
			EnterCriticalSection( &pUsbFps->BulkIn.Lock );
			EnterCriticalSection( &pUsbFps->BulkOut.Lock );

			pUsbFps->Flags.Open = FALSE;
			pUsbFps->Flags.UnloadPending = TRUE;

			LeaveCriticalSection( &pUsbFps->Control.Lock );
			LeaveCriticalSection( &pUsbFps->BulkIn.Lock );
			LeaveCriticalSection( &pUsbFps->BulkOut.Lock );
			
			DEBUGMSG(ZONE_USB_INIT, (TEXT("Waiting for CloseEvent...\n")));
			dwWaitReturn = WaitForSingleObject(pUsbFps->hCloseEvent,INFINITE);
			switch( dwWaitReturn )
			{
			case WAIT_OBJECT_0:
				DEBUGMSG(ZONE_USB_INIT,	(TEXT("...CloseEvent signalled\n")));
				break;
			case WAIT_FAILED:
				DEBUGMSG(ZONE_ERR,(TEXT("CloseEvent error:%d\n"),GetLastError()));
				break;
			default:
				DEBUGMSG(ZONE_ERR,(TEXT("Unhandled WaitReason:%d\n"), dwWaitReturn ));
				break;
			}
			// give it a chance to finish up...
			Sleep(1000);
		}
		DEBUGMSG( ZONE_USB_INIT, (TEXT("DeactivateDevice\n")));
		// unloads the driver for the Finger Print device, and delete
		// the active key of the device from the registry
		bRc = DeactivateDevice( pUsbFps->hStreamDevice );
		if ( !bRc )
		{
			DEBUGMSG(ZONE_ERR,(TEXT("DeactivateDevice error: %d\n"),GetLastError()));
		}
		// release the memory used by this device driver...
		RemoveDeviceContext( pUsbFps );
		return TRUE;
	default:
		DEBUGMSG( ZONE_ERR, (TEXT("Unhandled code:%d\n"), dwCode));
		break;
	}
	AddLog("DeviceNotify Error");
	DEBUGMSG( ZONE_USB_INIT, (TEXT("<DeviceNotify\n")));
	return FALSE;
}