
#include "stdafx.h"
#include "driver.h"
#include "usbdriver.h"

/*++
===============================================================================
Function WriteUsb
This function enables the internal state machine to transmit bytes via the
USB bulk out pipe.
Parameters
pUsbFps [in] Handle returned by the AES_Open function.
pBuffer [in] Pointer to the bytes to be written.
dwBufferLength [in] Specifies the number of bytes to be written.
Return Values
The number of bytes written indicates success. A value of 1
indicates failure.
===============================================================================
*/
DWORD ControlIO(PUSBFPS_CONTEXT pUsbFps, 
				_ControlRequest *  RequestData,
				PUCHAR pBuffer, DWORD dwBufferLength,				
				PDWORD pBytesActual)
{	
	AddLog("ControlIO");	

	DWORD dwBytesTransferred ;	
	DWORD dwErr = ERROR_SUCCESS;
	DWORD dwUsbErr = USB_NO_ERROR;
	DWORD Flags =0;	
	*pBytesActual=0;

	USB_DEVICE_REQUEST Request = {
		RequestData->RequestDirectionIN?USB_REQUEST_FOR_DEVICE|USB_REQUEST_VENDOR|USB_REQUEST_DEVICE_TO_HOST:USB_REQUEST_FOR_DEVICE|USB_REQUEST_VENDOR|USB_REQUEST_HOST_TO_DEVICE,
		(UCHAR)RequestData->bRequest,
		RequestData->wValue, 
		RequestData->wIndex, 
		(WORD) dwBufferLength};	

	if (!RequestData->RequestDirectionIN)
	{
		AddLog("ControlIO USB_OUT_TRANSFER");
		Flags = USB_OUT_TRANSFER;		
	}
	else
	{
		AddLog("ControlIO USB_IN_TRANSFER");
		Flags = USB_IN_TRANSFER | USB_SHORT_TRANSFER_OK;		
	}
		
	DEBUGMSG(ZONE_AES_WRITE, (TEXT(">ControlUsb(0x%x, %d)\n"),pBuffer, dwBufferLength));
	// is the device currently available?
	if ( ACCEPT_IO( pUsbFps ) )
	{
		AddLog("ControlIO ACCEPT_IO Accepted");
		// is there a buffer for transfer?
		do 
		{
			AddLog("ControlIO IssueVendorTransfer");	
			
			// try to read from the device...
			dwErr = IssueVendorTransfer(pUsbFps->lpUsbFuncs,
				pUsbFps->hUsbDevice,
				DefaultTransferComplete,
				pUsbFps->ControlPipeEvent,
				Flags,
				&Request,
				pBuffer, 0,				
				&dwBytesTransferred,
				pUsbFps->Control.Timeout,
				&dwUsbErr);

			// was an error detected?
			if ((ERROR_SUCCESS != dwErr) || (USB_NO_ERROR != dwUsbErr))
			{				
				AddLogMessage3("Error During ControlTransfer, dwErr=%d, dwUsbErr=%d, timeout=%d",dwErr,dwUsbErr, pUsbFps->Control.Timeout);								
				// the device was probably unplugged...
				if (ERROR_GEN_FAILURE == dwErr)
				{
					AddLog("ControlIO IssueVendorTransfer Error unplugged");
					// return failure...
					return(FALSE);
				}
			}

			*pBytesActual+=dwBytesTransferred;
			pBuffer+=dwBytesTransferred;
			dwBufferLength-=dwBytesTransferred;
		}while(pBuffer && dwBufferLength);	
	}
	else
	{
		AddLog("ControlUsb: ERROR_INVALID_HANDLE");
		DEBUGMSG( ZONE_ERR,(TEXT("ControlUsb: ERROR_INVALID_HANDLE\n")));
		SetLastError(ERROR_INVALID_HANDLE);
		return FALSE;
	}
	
	DEBUGMSG(ZONE_AES_WRITE,(TEXT("<ControlUsb:%d\n"), dwBytesTransferred ));	
	return TRUE;
}