#pragma once
#include <windows.h>
#include <types.h>
#include <excpt.h>
#include <tchar.h>
#include <devload.h>
#include <storemgr.h>
#include <wdm.h>
#include <ntcompat.h>
//
#include <usbdi.h>
#include <usbclient.h>
//-------------------------
#include "log.h"
#include "ioctl.h"

#define USB_LCARD_DEVICE_VENDOR_ID	0x0471
#define DEVICE_PREFIX TEXT("DEV")
#define CLASS_NAME_SZ TEXT("LCARD_ARM_Class")
#define CLIENT_REGKEY_SZ TEXT("Drivers\\USB\\ClientDrivers\\LCARD_ARM_Class")

#ifdef DEBUG
//
// Debug zones
//
#define ZONE_INIT       DEBUGZONE(0)
#define ZONE_LOADER     DEBUGZONE(1)
#define ZONE_CONFIG     DEBUGZONE(2)
#define ZONE_PIPE       DEBUGZONE(3)
#define ZONE_VENDOR     DEBUGZONE(4)
#define ZONE_CONTROL    DEBUGZONE(5)
#define ZONE_BULK       DEBUGZONE(6)
#define ZONE_INTERRUPT  DEBUGZONE(7)
#define ZONE_ISOCH      DEBUGZONE(8)
#define ZONE_API        DEBUGZONE(9)
#define ZONE_WARNING    DEBUGZONE(14)
#define ZONE_ERROR      DEBUGZONE(15)

#define ZONE_ERR		DEBUGZONE(16)
#define ZONE_WARN		DEBUGZONE(17)
#define ZONE_TRACE		DEBUGZONE(19)
#define ZONE_AES_INIT	DEBUGZONE(20)
#define ZONE_AES_READ	DEBUGZONE(21)
#define ZONE_AES_WRITE	DEBUGZONE(22)
#define ZONE_AES_IOCTL	DEBUGZONE(23)
#define ZONE_USB_INIT	DEBUGZONE(24)



#endif  // DEBUG

#define SupportedDevicesPIDs 0x0, 0x0154, 0x2121, 0x1010

#define USB_LCARD_ARM_SIG 0x46505321 // "DEV#"  

#define MANUAL_RESET_EVENT	TRUE
#define AUTO_RESET_EVENT	FALSE

// Is the pointer a valid context?
// It's valid if we can read our signature
#define VALID_CONTEXT( pContext ) \
	(pContext && (USB_LCARD_ARM_SIG == pContext->ulSignature))
// Can the device accept any I/O requests?
// It can accept I/O IF:
// it's a valid context AND
// the driver is open AND
// the device is still plugged in
#define ACCEPT_IO( pContext ) \
	( VALID_CONTEXT( pContext ) && \
	pContext->Flags.Open && \
	!pContext->Flags.UnloadPending )


#pragma pack(1)
typedef struct 
{
	WORD RequestDirectionIN; // 1 -   0 -  
	WORD bRequest;
	WORD wValue;
	WORD wIndex;
}_ControlRequest;
#pragma pack()


// structure used to indicate driver status:
typedef struct _FLAGS
{
	UCHAR Open : 1; // bits 0
	UCHAR UnloadPending : 1; // bits 1
	UCHAR Reserved : 6; // bits 27
} FLAGS, *PFLAGS;
// Pipe information:
typedef struct _PIPE
{
	USB_PIPE hPipe; // USB Pipe handle
	HANDLE hEvent; // Completion Event
	UCHAR bIndex; // Endpoint's Address
	DWORD maxPacketSize;
	CRITICAL_SECTION Lock;
	DWORD Timeout;
} PIPE, *PPIPE;

// Everything that you ever wanted to know about a USB interface
// to a finger print sensor is contained in this structure:
typedef struct _USBFPS_CONTEXT
{
	ULONG ulSignature; // sanity check
	FLAGS Flags; // open/unloading status		
	HANDLE hStreamDevice; // Handle for Stream interface.
	HANDLE hUsbDevice; // USB handle to the device
	HANDLE hCloseEvent; // Signals the device has been closed.
	LPCUSB_FUNCS lpUsbFuncs; // USBD Function table
	PIPE BulkOut; // Bulk OUT Pipe
	PIPE BulkIn; // Bulk IN Pipe	
	PIPE Control; // Bulk IN Pipe	
	HANDLE ControlPipeEvent; // Completion Event	
} USBFPS_CONTEXT, *PUSBFPS_CONTEXT;

//======================================
VOID RemoveDeviceContext(PUSBFPS_CONTEXT pUsbFps);
BOOL WINAPI DeviceNotify(LPVOID lpvNotifyParameter,
						 DWORD dwCode,
						 LPDWORD *lpdwInfo1,
						 LPDWORD *lpdwInfo2,
						 LPDWORD *lpdwInfo3,
						 LPDWORD *lpdwInfo4);
BOOL SetUsbInterface(PUSBFPS_CONTEXT pUsbFps,  LPCWSTR UniqueDriverId);
//---------------
DWORD ControlIO(PUSBFPS_CONTEXT pUsbFps, 
				_ControlRequest * RequestData,
				PUCHAR pBuffer, DWORD dwBufferLength,
				PDWORD pBytesActual);
//======================================

#pragma pack(1)
typedef struct DEVICE_INITIAL_INFO
{
	ULONG		Base;
	ULONG		BaseL;
	ULONG		Base1;
	ULONG		BaseL1;
	ULONG		Mem;
	ULONG		MemL;
	ULONG		Mem1;
	ULONG		MemL1;
	ULONG		Irq;
	ULONG		DeviceID;
	ULONG		DSPType;
	ULONG		Dma;
	ULONG		DmaDac;
	ULONG		DTA_REG;
	ULONG		IDMA_REG;
	ULONG		CMD_REG;
	ULONG		IRQ_RST;
	ULONG		DTA_ARRAY;
	ULONG		RDY_REG;
	ULONG		CFG_REG;
}S_DEVICE_INITIAL_INFO;
#pragma pack()

typedef enum 	
{
			E14_440_ID	= 30,
			E14_140_ID	= 31,
		    E2010_ID		= 32,
			E270_ID		= 33,
			LTR010_ID	= 36,
			LTR021_ID	= 37,
			E154_ID		= 38,

			ENUM_ALL_USB_DEVICE_ID = 0xFFFE,
			INVALID_USB_DEVICE_ID  = 0xFFFF
}E_Supported_Device_ID;