#include "stdafx.h"
//------------------------------------------------------------------------------
#include <algorithm>
#include <stdio.h>
#include "e140.h"
#include "ErrorBaseIds.h"
//------------------------------------------------------------------------------

#define 	LAST_ERROR_NUMBER(ErrorNumber)	LastErrorNumber = E140_BASE_ERROR_ID + ErrorNumber

// -----------------------------------------------------------------------------
//  
// ----------------------------------------------------------------------------
TLE140::TLE140(HINSTANCE hInst) : TLUSBBASE(hInst)
{
	//     
	ZeroMemory(&ModuleDescriptor, sizeof(MODULE_DESCRIPTOR));
	//     
	AdcPars.AdcRate = 100.0;
	AdcPars.ChannelsQuantity = 0x1;
	ZeroMemory(AdcPars.ControlTable, sizeof(AdcPars.ControlTable));
	//    
	IsDataAcquisitionInProgress = FALSE;
	//    Firmware 
   FirmwareVersionNumber = 0.0;
}

// -----------------------------------------------------------------------------
//  
// -----------------------------------------------------------------------------
TLE140::~TLE140() { }



// =============================================================================
//        14-140
// =============================================================================
// -----------------------------------------------------------------------------
//        USB 
// -----------------------------------------------------------------------------
BOOL WINAPI TLE140::OpenLDevice (WORD VirtualSlot)
{
	char ModuleName[16];

	//         USB 
	if(!TLUSBBASE::OpenLDeviceByID(VirtualSlot, E14_140_ID)) { return FALSE; }
	//    
	else if(!TLUSBBASE::GetModuleName(ModuleName)) { TLUSBBASE::CloseLDevice(); return FALSE; }
	// ,    E14-140
	else if(strcmp(ModuleName, "E140")) { TLUSBBASE::CloseLDevice(); LAST_ERROR_NUMBER(0); return FALSE; }
	//  
	else return TRUE;
}

//------------------------------------------------------------------------------------
//    
//------------------------------------------------------------------------------------
BOOL WINAPI TLE140::CloseLDevice(void)
{
	//  
	if(!TLUSBBASE::CloseLDevice()) return FALSE;
	//   :)))))
	else return TRUE;
}

//------------------------------------------------------------------------------------
//    
//------------------------------------------------------------------------------------
BOOL WINAPI TLE140::ReleaseLInstance(void)
{
	BOOL Status = TRUE;

	//    
	if(!CloseLDevice()) Status = FALSE;
	//    
	delete this;
	//    
	return Status;
}

//------------------------------------------------------------------------------
//   
//------------------------------------------------------------------------------
HANDLE WINAPI TLE140::GetModuleHandle(void) { return hDevice; }

//------------------------------------------------------------------------------
//    
//------------------------------------------------------------------------------
BOOL WINAPI TLE140::GetModuleName(PCHAR const ModuleName)
{
	//  
	if(!ModuleName) { LAST_ERROR_NUMBER(1); return FALSE; }
	//     
	else if(!TLUSBBASE::GetModuleName(ModuleName)) return FALSE;
	//   :)))))
	else return TRUE;
}

//------------------------------------------------------------------------------------
//       USB
//------------------------------------------------------------------------------------
BOOL WINAPI TLE140::GetUsbSpeed(BYTE * const UsbSpeed)
{
	//  
	if(!UsbSpeed) { LAST_ERROR_NUMBER(1); return FALSE; }
	//      USB 
	else if(!TLUSBBASE::GetUsbSpeed(UsbSpeed)) return FALSE;
	//   :)))))
	else return TRUE;
}

//------------------------------------------------------------------------------
//      
//------------------------------------------------------------------------------
BOOL WINAPI TLE140::GetLastErrorInfo(LAST_ERROR_INFO_LUSBAPI * const LastErrorInfo)
{
	return TLUSBBASE::GetLastErrorInfo(LastErrorInfo);
}




// =============================================================================
//      
// =============================================================================
// -----------------------------------------------------------------------------
//     
// -----------------------------------------------------------------------------
BOOL WINAPI TLE140::GET_ADC_PARS(ADC_PARS_E140 * const AdcPars)
{
	BYTE b, smode;
	BYTE buf[136];
	WORD i;
	DWORD FREQ;

	//  
	if(!AdcPars) { LAST_ERROR_NUMBER(1); return FALSE; }
	//     
	if(!GetArray(buf, sizeof(buf), ADC_PARS_BASE)) { LAST_ERROR_NUMBER(2); return FALSE; }
	//   
	for(i = 0x0; i < MAX_CONTROL_TABLE_LENGTH_E140; i++)
	{
		b = buf[i];
		this->AdcPars.ControlTable[i] = (WORD)((b & 0xF0) | ((b << 2) & 0x0C) | ((b >> 2) & 0x03));
	}
	FREQ = ModuleDescriptor.QuartzFrequency;
	if((FREQ > 24000000L) || (FREQ < 1000000L)) FREQ = 16000000L;
	//   ()
	this->AdcPars.AdcRate = (double)FREQ/2000.0/(*(WORD*)(buf+128));
	//    
	this->AdcPars.ChannelsQuantity = buf[130];
	//   ()
	this->AdcPars.InterKadrDelay = (double)((WORD)buf[131] + 1.0)/this->AdcPars.AdcRate;
	//   ()
	this->AdcPars.KadrRate = (double)this->AdcPars.AdcRate/((WORD)buf[130] + (WORD)buf[131]);
	//  
	smode = (BYTE)(buf[132] & 0x0F);
	if(smode == 0x1)      	this->AdcPars.InputMode = 2;
	else if(smode == 0x2) 	this->AdcPars.InputMode = 1;
	else if(smode > 0x2) 	this->AdcPars.InputMode = 3;
	else              		this->AdcPars.InputMode = 0;
	this->AdcPars.SynchroAdType = (BYTE)((smode == 0x5 || smode == 0x6) ? 0x1 : 0x0);
	this->AdcPars.SynchroAdMode = (BYTE)((smode == 0x4 || smode == 0x6) ? 0x1 : 0x0);
	//   
	this->AdcPars.ClkSource = (BYTE)((buf[132] >> 0x6) & 0x1);
	//       
	this->AdcPars.EnableClkOutput = (BYTE)((buf[132] >> 0x7) & 0x1);
	//      
	b = buf[133];
	this->AdcPars.SynchroAdChannel = (WORD)((b & 0xF0) | ((b << 0x2) & 0x0C) | ((b >> 0x2) & 0x03));
	//     
	this->AdcPars.SynchroAdPorog = *(SHORT*)(buf+134);
	//
	*AdcPars = this->AdcPars;
	//   :)))))
	return TRUE;
}

// -----------------------------------------------------------------------------
//      
// -----------------------------------------------------------------------------
BOOL WINAPI TLE140::SET_ADC_PARS(ADC_PARS_E140 * const AdcPars)
{
	BYTE b, smode;
	BYTE buf[136];
	DWORD i;
	DWORD FREQ;

	//  
	if(!AdcPars) { LAST_ERROR_NUMBER(1); return FALSE; }
	//     -        
	//     STOP_ADC()
	else if(IsDataAcquisitionInProgress) { LAST_ERROR_NUMBER(3); return FALSE; }

	//  
	ZeroMemory(buf, sizeof(buf));
	//   
	for(i = 0x0; i < MAX_CONTROL_TABLE_LENGTH_E140; i++)
	{
		b = (BYTE)AdcPars->ControlTable[i];
		buf[i] = (BYTE)((b & 0xF0) | ((b << 0x2) & 0x0C) | ((b >> 0x2) & 0x03));
	}
	FREQ = ModuleDescriptor.QuartzFrequency;
	if((FREQ > 24000000L) || (FREQ < 1000000L)) FREQ = 16000000L;
	//     
	i = (DWORD)((double)FREQ/2000/AdcPars->AdcRate + 0.5);
	if(i > 0xFFFF) i = 0xFFFF;
	else if(i < 80) i = 80;
	AdcPars->AdcRate = (double)FREQ/2000/i;
	*(WORD*)(buf+128) = (WORD)i;
	//    
	if(!AdcPars->ChannelsQuantity) { LAST_ERROR_NUMBER(26); return FALSE; }
	else if(AdcPars->ChannelsQuantity > MAX_CONTROL_TABLE_LENGTH_E140) AdcPars->ChannelsQuantity = MAX_CONTROL_TABLE_LENGTH_E140;
	buf[130] = (BYTE)AdcPars->ChannelsQuantity;
	//   ()
	i = (DWORD)(AdcPars->AdcRate*AdcPars->InterKadrDelay + 0.5);
	if(i == 0) i = 1;
	else if(i > 256) i = 256;
	buf[131] = (BYTE)(i - 0x1);
	AdcPars->InterKadrDelay = i/AdcPars->AdcRate;
	//   ()
	AdcPars->KadrRate = (double)AdcPars->AdcRate/((WORD)buf[130] + (WORD)buf[131]);
    //  
	if(AdcPars->InputMode > ANALOG_SYNC_E140) AdcPars->InputMode = ANALOG_SYNC_E140;
	smode = (BYTE)AdcPars->InputMode;
	if(smode == 1) smode=2;
	else if(smode == 2) smode = (BYTE)(0x1);
	else if(smode == 3)
	{
		if(AdcPars->SynchroAdType) smode += (BYTE)(0x2);
		if(AdcPars->SynchroAdMode) smode += (BYTE)(0x1);
	}
	buf[132] = smode;
	//   
	if(AdcPars->ClkSource) buf[132] |= (0x1 << 0x6);
	//       
	if(AdcPars->EnableClkOutput) buf[132] |= (0x1 << 0x7);
	//      
	b = (BYTE)AdcPars->SynchroAdChannel;
	buf[133] = (BYTE)((b & 0xF0) | ((b << 0x2) & 0x0C) | ((b >> 0x2) & 0x03));
	//     
	*(SHORT*)(buf+134) = AdcPars->SynchroAdPorog;
	//     
	this->AdcPars=*AdcPars;
	//   
	if(!PutArray(buf, sizeof(buf), ADC_PARS_BASE)) { LAST_ERROR_NUMBER(4); return FALSE; }
	//   :)))))
	else return TRUE;
}

// -----------------------------------------------------------------------------
//   
// -----------------------------------------------------------------------------
BOOL WINAPI TLE140::START_ADC(void)
{
	WORD InBuf[4] = { 0x0000, V_START_ADC, 0x0, 0x0 };

	//     
	if(!TLUSBBASE::LDeviceIoControl(IOCTL_CONTROL_PIPE_VENDOR, &InBuf, sizeof(InBuf), NULL, 0, TimeOut)) { LAST_ERROR_NUMBER(5); return FALSE; }
	//    
	IsDataAcquisitionInProgress = TRUE;
	//  
	return TRUE;
}

// -----------------------------------------------------------------------------
//  
// -----------------------------------------------------------------------------
BOOL WINAPI TLE140::STOP_ADC(void)
{
	WORD InBuf[4] = {0x0000, V_STOP_ADC, 0x0, 0x0 };

	//     
	if(!TLUSBBASE::LDeviceIoControl(IOCTL_CONTROL_PIPE_VENDOR, &InBuf, sizeof(InBuf), NULL, 0, TimeOut)) { LAST_ERROR_NUMBER(5); return FALSE; }
	//    
	else if(!TLUSBBASE::LDeviceIoControl(IOCTL_CONTROL_RESET_BULK_IN_PIPE/*reset Read Pipe*/, NULL, NULL, NULL, NULL, TimeOut)) { LAST_ERROR_NUMBER(5); return FALSE; }
	//    
	IsDataAcquisitionInProgress = FALSE;
	//  
	return TRUE;
}

// -----------------------------------------------------------------------------
//       
// -----------------------------------------------------------------------------
BOOL WINAPI TLE140::ADC_KADR(SHORT * const Data)
{
	//       
	WORD InBuf[4] = { 0x0000, V_START_ADC_ONCE, 0x0, 0x0};
	// -     
	DWORD PointsToRead;
	//       
	IO_REQUEST_LUSBAPI IoReq;

	//  
	if(!Data) { LAST_ERROR_NUMBER(1); return FALSE; }
	//  -  
	else if(AdcPars.ChannelsQuantity > 32) { LAST_ERROR_NUMBER(6); return FALSE; }
	//     -        
	//     STOP_ADC()
	else if(IsDataAcquisitionInProgress) { LAST_ERROR_NUMBER(3); return FALSE; }
	//    STOP_ADC()
	else if(!STOP_ADC()) { LAST_ERROR_NUMBER(25); return FALSE; }
	//   
//	else if(!TLUSBBASE::LDeviceIoControl(DIOC_RESET_PIPE3, NULL, NULL, NULL, NULL, TimeOut)) { LAST_ERROR_NUMBER(5); return FALSE; }
	//      
	else if(!TLUSBBASE::LDeviceIoControl(IOCTL_CONTROL_PIPE_VENDOR, &InBuf, sizeof(InBuf), NULL, 0, TimeOut)) { LAST_ERROR_NUMBER(5); return FALSE; }

	//     IO_REQUEST_LUSBAPI   
	if(!(AdcPars.ChannelsQuantity%32)) PointsToRead = AdcPars.ChannelsQuantity;
	else { PointsToRead = AdcPars.ChannelsQuantity - AdcPars.ChannelsQuantity%32; PointsToRead += 32; }
	//        
	IoReq.Buffer 					= new SHORT[PointsToRead];	//  
	if(!IoReq.Buffer) { LAST_ERROR_NUMBER(7); return FALSE; }
	IoReq.NumberOfWordsToPass 	= PointsToRead;	 			// -   SHORT
	IoReq.NumberOfWordsPassed 	= 0x0;
	IoReq.Overlapped			  	= NULL;				 			//     
	IoReq.TimeOut					= (DWORD)(PointsToRead/AdcPars.AdcRate + 1000);	//   
	//    
	if(!ReadData(&IoReq)) { LAST_ERROR_NUMBER(8); delete [] IoReq.Buffer; return FALSE; }
	//   
   CopyMemory(Data, &IoReq.Buffer[(32 - AdcPars.ChannelsQuantity) & 0x1F], AdcPars.ChannelsQuantity*sizeof(SHORT));
	//  
	if(IoReq.Buffer) delete [] IoReq.Buffer;
	//  
	return TRUE;
}


// -----------------------------------------------------------------------------
//         
// -----------------------------------------------------------------------------
BOOL WINAPI TLE140::ADC_SAMPLE(SHORT * const Data, WORD Channel)
{
	WORD b = (WORD)((Channel & 0xF0) | ((Channel << 0x2) & 0x0C) | ((Channel >> 0x2) & 0x03));

	//     -        
	//     STOP_ADC()
	if(IsDataAcquisitionInProgress) { LAST_ERROR_NUMBER(3); return FALSE; }
	//   
	else if(!PutArray((BYTE*)&b, 0x2, ADC_CHANNEL_SELECT_REGISTER)) { LAST_ERROR_NUMBER(4); return FALSE; }
	// 
	Sleep(40);
	// 2  
	if(!PutArray((BYTE*)&b, 0x2, ADC_CHANNEL_SELECT_REGISTER)) { LAST_ERROR_NUMBER(4); return FALSE; }
	//  
	else if(!GetArray((BYTE*)Data, 0x2, ADC_DATA_REGISTER)) { LAST_ERROR_NUMBER(2); return FALSE; }
	//  
	else return TRUE;
}

// -----------------------------------------------------------------------------
//       
// -----------------------------------------------------------------------------
BOOL WINAPI TLE140::ReadData(IO_REQUEST_LUSBAPI * const ReadRequest)
{
	DWORD NumberOfBytesRead;

	//   
	if(!ReadRequest->Buffer) { LAST_ERROR_NUMBER(1); return FALSE; }
	//        0x20(32)
	else if(ReadRequest->NumberOfWordsToPass < 0x20) ReadRequest->NumberOfWordsToPass = 0x20;
	//    0x100000(1024*1024)
	else if(ReadRequest->NumberOfWordsToPass > (1024*1024)) ReadRequest->NumberOfWordsToPass = 1024*1024;
	//       0x20(32)
	ReadRequest->NumberOfWordsToPass -= ReadRequest->NumberOfWordsToPass%32;
	//
	NumberOfBytesRead = 0x0;

	
	if(ReadRequest->Overlapped)
	{
		ReadRequest->TimeOut=INFINITE;
	}

	if (!DeviceIoControl(hDevice, IOCTL_CONTROL_SET_READ_BULK_TIMEOUT_MS, &ReadRequest->TimeOut, sizeof(DWORD), NULL,0,NULL,NULL))
	{ LAST_ERROR_NUMBER(11);return FALSE;};

	//       - 
	NumberOfBytesRead = 0x0;
	if(!ReadFile(hDevice, ReadRequest->Buffer, 2*ReadRequest->NumberOfWordsToPass, &NumberOfBytesRead, NULL))
	{ LAST_ERROR_NUMBER(9); return FALSE; };		
	// -  
	ReadRequest->NumberOfWordsPassed = NumberOfBytesRead/2;
	//       ( )   - 
	if(ReadRequest->NumberOfWordsToPass != ReadRequest->NumberOfWordsPassed) { LAST_ERROR_NUMBER(14); return FALSE; }

	//   :)))))
	return TRUE;
}




// =============================================================================
//       
// =============================================================================
// -----------------------------------------------------------------------------
//       
// -----------------------------------------------------------------------------
BOOL WINAPI TLE140::DAC_SAMPLE(SHORT * const DacData, WORD DacChannel)
{
	SHORT DacParam;

	//        
	if(ModuleDescriptor.IsDacPresented == DAC_INACCESSIBLED_E140) { LAST_ERROR_NUMBER(15); return FALSE; }
	//    
	else if(DacChannel >= DAC_CHANNELS_QUANTITY_E140) { LAST_ERROR_NUMBER(16); return FALSE; }

	//      
	if((*DacData) < -2048) *DacData = -2048;
	else if((*DacData) > 2047) *DacData = 2047;
	//       AVR
	DacParam = (SHORT)((*DacData) & 0xFFF);
	DacParam |= (SHORT)(DacChannel << 12);
	//   
	if(!PutArray((BYTE*)&DacParam, 0x2, DAC_DATA_REGISTER)) { LAST_ERROR_NUMBER(4); return FALSE; }
	//  
	else return TRUE;
}




// =============================================================================
//           
// =============================================================================
// -----------------------------------------------------------------------------
//        
// -----------------------------------------------------------------------------
BOOL WINAPI TLE140::ENABLE_TTL_OUT(BOOL EnableTtlOut)
{
	BYTE Flag = (BYTE)(EnableTtlOut ? 0x1 : 0x0);

	//     
	if(!PutArray(&Flag, 0x1, DOUT_ENABLE_REGISTER)) { LAST_ERROR_NUMBER(4); return FALSE; }
	//  
	else return TRUE;
}

// -----------------------------------------------------------------------------
//         
// -----------------------------------------------------------------------------
BOOL WINAPI TLE140::TTL_IN(WORD * const TtlIn)
{
	//    
	if(!GetArray((BYTE *)TtlIn, 0x2, DIN_REGISTER)) { LAST_ERROR_NUMBER(2); return FALSE; }
	//  
	else return TRUE;
}

// -----------------------------------------------------------------------------
//         
// -----------------------------------------------------------------------------
BOOL WINAPI TLE140::TTL_OUT(WORD TtlOut)
{
	//    
	if(!PutArray((BYTE*)&TtlOut, 0x2, DOUT_REGISTER)) { LAST_ERROR_NUMBER(4); return FALSE; }
	//  
	else return TRUE;
}




// =============================================================================
//        
// =============================================================================
// -----------------------------------------------------------------------------
// /      
// -----------------------------------------------------------------------------
BOOL WINAPI TLE140::ENABLE_FLASH_WRITE(BOOL IsUserFlashWriteEnabled)
{
	BYTE Flag = (BYTE)(IsUserFlashWriteEnabled ? 0x1 : 0x0);

	//   /   
	if(!PutArray(&Flag, 0x1, ENABLE_FLASH_WRITE_FLAG)) { LAST_ERROR_NUMBER(4); return FALSE; }
	//  
	TLE140::IsUserFlashWriteEnabled = IsUserFlashWriteEnabled;
	//  
	return TRUE;
}

// -----------------------------------------------------------------------------
//      
// -----------------------------------------------------------------------------
BOOL WINAPI TLE140::READ_FLASH_WORD(WORD FlashAddress, SHORT * const FlashWord)
{
	//      [0..255]
	if(FlashAddress >= USER_FLASH_SIZE_E140/2) { LAST_ERROR_NUMBER(17); return FALSE; }
	//     
	else if(!GetArray((BYTE*)FlashWord, 0x2, (WORD)(DATA_FLASH_BASE + (FlashAddress << 0x1)))) { LAST_ERROR_NUMBER(2); return FALSE; }
	//  
	else return TRUE;
}

// -----------------------------------------------------------------------------
//      
// -----------------------------------------------------------------------------
BOOL WINAPI TLE140::WRITE_FLASH_WORD(WORD FlashAddress, SHORT FlashWord)
{
	//     -          
	//     STOP_ADC()
	if(IsDataAcquisitionInProgress) { LAST_ERROR_NUMBER(3); return FALSE; }
	//    
	else if(!IsUserFlashWriteEnabled) { LAST_ERROR_NUMBER(18); return FALSE; }
	//      [0..255]
	else if(FlashAddress >= USER_FLASH_SIZE_E140/2)  { LAST_ERROR_NUMBER(17); return FALSE; }
	//     
	else if(!PutArray((BYTE*)&FlashWord, 0x2, (WORD)(DATA_FLASH_BASE + (FlashAddress << 0x1)))) { LAST_ERROR_NUMBER(4); return FALSE; }
	//  
	else return TRUE;
}





// =============================================================================
//        
// =============================================================================
// -----------------------------------------------------------------------------
//        
// -----------------------------------------------------------------------------
BOOL WINAPI TLE140::GET_MODULE_DESCRIPTION(MODULE_DESCRIPTION_E140 * const md)
{
	char *pTag;
	BYTE Buffer[128];
	BYTE Str[17];
	WORD i;

	//  
	if(!md) { LAST_ERROR_NUMBER(1); return FALSE; }
	//      
	else if(!GetArray(Buffer, sizeof(Buffer), DESCRIPTOR_BASE)) { LAST_ERROR_NUMBER(2); return FALSE; }
	//   
	else if(!UnpackModuleDescriptor(Buffer, &ModuleDescriptor)) { LAST_ERROR_NUMBER(19); return FALSE; }

	// Module Info
	strncpy((char *)md->Module.CompanyName,  			"L-Card Ltd.",    								sizeof(md->Module.CompanyName));
	if(!strcmp((char *)ModuleDescriptor.Name, 		"E140")) strcpy((char *)md->Module.DeviceName, "E14-140");
	else strncpy((char *)md->Module.DeviceName,		(char *)ModuleDescriptor.Name,	 		min(sizeof(md->Module.DeviceName), sizeof(ModuleDescriptor.Name)));
	strncpy((char *)md->Module.SerialNumber, 			(char *)ModuleDescriptor.SerialNumber,	min(sizeof(md->Module.SerialNumber), sizeof(ModuleDescriptor.SerialNumber)));
	strcpy((char *)md->Module.Comment,      			"  //     USB 1.1");
	md->Module.Revision = ModuleDescriptor.Revision;

	// Interface Info
	md->Interface.Active = TRUE;
	strcpy((char *)md->Interface.Name, "USB 1.1");
	strcpy((char *)md->Interface.Comment, "Full-Speed Mode (12 Mbit/s)");

	// MCU Info
	if(!GetArray(Buffer, 0x10, LBIOS_VERSION)) { LAST_ERROR_NUMBER(2); return FALSE; }
	md->Mcu.Active = TRUE;
	strcpy((char *)md->Mcu.Name, "AVR ATmega8515");
	sprintf((char *)Str, "%1u.%1u", Buffer[1], Buffer[0]);
	strncpy((char *)md->Mcu.Version.Version,  		(char *)Str, 		min(sizeof(md->Mcu.Version.Version), sizeof(Str)));
	strncpy((char *)md->Mcu.Version.Date,  			(char *)(Buffer+2), 14);
	strcpy((char *)md->Mcu.Version.Manufacturer,  	"L-Card Ltd.");
	strcpy((char *)md->Mcu.Version.Author,  			"Kodorkin A.V.");
	strncpy((char *)md->Mcu.Version.Comment,			"    E14-140", sizeof(md->Mcu.Version.Comment));
	strncpy((char *)md->Mcu.Comment, 					"8-bit Microcontroller with 8K Bytes In-System Programmable Flash", sizeof(md->Mcu.Comment));
	md->Mcu.ClockRate = ModuleDescriptor.QuartzFrequency/1000.0;		//  
	//     Firmware 
	FirmwareVersionNumber = strtod((char *)md->Mcu.Version.Version, &pTag);

	// ADC Info
	md->Adc.Active = TRUE;
	strcpy((char *)md->Adc.Name, "LTC1416");
	//    i
	for(i = 0x0; i < ADC_CALIBR_COEFS_QUANTITY_E140; i++)
	{
	   md->Adc.OffsetCalibration[i] = ModuleDescriptor.AdcOffsetCoefs[i];
	   md->Adc.ScaleCalibration[i] = ModuleDescriptor.AdcScaleCoefs[i];
	}
	strcpy((char *)md->Adc.Comment, "14-Bit 400Ksps ADC converter");

	// DAC Info
	if(ModuleDescriptor.IsDacPresented)
	{
		md->Dac.Active = TRUE;
		strcpy((char *)md->Dac.Name, "AD7249");
		for(i = 0x0; i < DAC_CALIBR_COEFS_QUANTITY_E140; i++)
		{
			md->Dac.OffsetCalibration[i] = ModuleDescriptor.DacOffsetCoefs[i];
			md->Dac.ScaleCalibration[i] = ModuleDescriptor.DacScaleCoefs[i];
		}
		strcpy((char *)md->Dac.Comment, "12-Bit 125Ksps DAC converter");
	}
	else md->Dac.Active = FALSE;

	// IO Info
	md->DigitalIo.Active = TRUE;
	strcpy((char *)md->DigitalIo.Name, "74HC165/74HC574");
	md->DigitalIo.InLinesQuantity = 16;
	md->DigitalIo.OutLinesQuantity = 16;
	strcpy((char *)md->DigitalIo.Comment, "TTL/CMOS compatible");

	//  
	return TRUE;
}
// -----------------------------------------------------------------------------
//        
// -----------------------------------------------------------------------------
BOOL WINAPI WINAPI TLE140::SAVE_MODULE_DESCRIPTION(MODULE_DESCRIPTION_E140 * const md)
{
	BYTE Buffer[128];
	WORD i;

	strcpy((char *)ModuleDescriptor.Name, 				"E140");
//	strncpy((char *)ModuleDescriptor.Name, 			(char *)md->Module.DeviceName,	min(sizeof(md->Module.DeviceName), sizeof(ModuleDescriptor.Name)));
	strncpy((char *)ModuleDescriptor.SerialNumber,	(char *)md->Module.SerialNumber, min(sizeof(md->Module.SerialNumber), sizeof(ModuleDescriptor.SerialNumber)));

	//    
	if((	md->Module.Revision > REVISIONS_E140[REVISIONS_QUANTITY_E140 - 0x1] &&
			md->Module.Revision <= 'Z')) { LAST_ERROR_NUMBER(23); return FALSE; }
	ModuleDescriptor.Revision = md->Module.Revision;

	ModuleDescriptor.IsDacPresented = (BYTE)(md->Dac.Active ? 0x1 : 0x0);
	ModuleDescriptor.QuartzFrequency = (long)(1000.0*md->Mcu.ClockRate + 0.5);		//  

	//   
	for(i = 0x0; i < ADC_CALIBR_COEFS_QUANTITY_E140; i++)
	{
		//     i
		if((fabs(md->Adc.OffsetCalibration[i]) > 8191.0) ||
        	(fabs(md->Adc.OffsetCalibration[i]) < FloatEps)) ModuleDescriptor.AdcOffsetCoefs[i] = 0.0;
		else ModuleDescriptor.AdcOffsetCoefs[i] = md->Adc.OffsetCalibration[i];
		//     i
		if((fabs(md->Adc.ScaleCalibration[i]) > 2.0) ||
        	(fabs(md->Adc.ScaleCalibration[i]) < FloatEps)) ModuleDescriptor.AdcScaleCoefs[i] = 1.0;
		else ModuleDescriptor.AdcScaleCoefs[i] = md->Adc.ScaleCalibration[i];
	}
	//   
	for(i = 0x0; i < DAC_CALIBR_COEFS_QUANTITY_E140; i++)
	{
		//  -     
		if(md->Dac.Active)
		{
			//    i
			if((fabs(md->Dac.OffsetCalibration[i]) > 2047.0) ||
         	(fabs(md->Dac.OffsetCalibration[i]) < FloatEps)) ModuleDescriptor.DacOffsetCoefs[i] = 0.0;
			else ModuleDescriptor.DacOffsetCoefs[i] = md->Dac.OffsetCalibration[i];
			//    i
			if((fabs(md->Dac.ScaleCalibration[i]) > 2.0) ||
         	(fabs(md->Dac.ScaleCalibration[i]) < FloatEps)) ModuleDescriptor.DacScaleCoefs[i] = 1.0;
			else ModuleDescriptor.DacScaleCoefs[i] = md->Dac.ScaleCalibration[i];
		}
		else
		{
			//    i
			ModuleDescriptor.DacOffsetCoefs[i] = 0.0;
			//    i
			ModuleDescriptor.DacScaleCoefs[i] = 1.0;
		}
	}

	//  
	if(!md) { LAST_ERROR_NUMBER(1); return FALSE; }
	//   
	else if(!PackModuleDescriptor(Buffer, &ModuleDescriptor)) { LAST_ERROR_NUMBER(20); return FALSE; }
	//     
	if(!ENABLE_FLASH_WRITE(TRUE)) { LAST_ERROR_NUMBER(21); return FALSE; }
	//     
	else if(!PutArray(Buffer, sizeof(Buffer), DESCRIPTOR_BASE)) { LAST_ERROR_NUMBER(4); ENABLE_FLASH_WRITE(FALSE); return FALSE; }
	//     
	else if(!ENABLE_FLASH_WRITE(FALSE)) { LAST_ERROR_NUMBER(21); return FALSE; }
	//   
	else if(!UnpackModuleDescriptor(Buffer, &ModuleDescriptor)) { LAST_ERROR_NUMBER(19); return FALSE; }
	//  
	else return TRUE;
}





// =============================================================================
//       
// =============================================================================
// -----------------------------------------------------------------------------
//         USB   suspend
// -----------------------------------------------------------------------------
BOOL TLE140::SetSuspendModeFlag(BOOL SuspendModeFlag)
{
	BYTE Flag = (BYTE)(SuspendModeFlag ? 0x1 : 0x0);

	if(!PutArray(&Flag, 0x1, SUSPEND_MODE_FLAG)) { LAST_ERROR_NUMBER(4); return FALSE; }
	//  
	else return TRUE;
}

// -----------------------------------------------------------------------------
//      
// -----------------------------------------------------------------------------
BOOL TLE140::GetArray(BYTE * const Buffer, WORD Size, WORD Address)
{
	DWORD i;
	WORD InBuf[4] = { 0x0001, V_GET_ARRAY, Address, 0x0 };

	//    
	if(!Buffer || !Size) { LAST_ERROR_NUMBER(1); return FALSE; }
	//  
	for(i = 0x0; i < (DWORD)Size; i += MAX_USB_CONTROL_PIPE_BLOCK)
	{
		InBuf[0x2] = (WORD)(Address + i);
		if(!TLUSBBASE::LDeviceIoControl(IOCTL_CONTROL_PIPE_VENDOR,
                            &InBuf, sizeof(InBuf),
                            Buffer + i, min( Size - i, (DWORD)MAX_USB_CONTROL_PIPE_BLOCK),
                            TimeOut))  { LAST_ERROR_NUMBER(5); return FALSE; }
	}
	//   :)))))
	return TRUE;
}

// -----------------------------------------------------------------------------
//      
// -----------------------------------------------------------------------------
BOOL TLE140::PutArray(BYTE * const Buffer, WORD Size, WORD Address)
{
	DWORD i;
	WORD InBuf[4] = { 0x0000, V_PUT_ARRAY, Address, 0x0 };

	//    
	if(!Buffer || !Size) { LAST_ERROR_NUMBER(1); return FALSE; }
	//  
	for(i = 0x0; i < (DWORD)Size; i += MAX_USB_CONTROL_PIPE_BLOCK)
	{
		if(!TLUSBBASE::LDeviceIoControl(IOCTL_CONTROL_PIPE_VENDOR,
                            &InBuf, sizeof(InBuf),
                            Buffer + i, min(Size - i, (DWORD)MAX_USB_CONTROL_PIPE_BLOCK),
                            TimeOut)) { LAST_ERROR_NUMBER(5); return FALSE; }
	}
	//   :)))))
	return TRUE;
}

// -----------------------------------------------------------------------------
//   8
// -----------------------------------------------------------------------------
#define CRC8INIT         0x55
BYTE TLE140::CRC8ADD(BYTE a, BYTE b)
{
	return (BYTE)(a + b);
}
//
BYTE TLE140::CRC8CALC(BYTE *Buffer, WORD Size)
{
	BYTE crc = CRC8INIT;
	while(Size--) crc = CRC8ADD(crc, *Buffer++);
	return crc;
}

// -----------------------------------------------------------------------------
//    
// -----------------------------------------------------------------------------
BOOL TLE140::PackModuleDescriptor(BYTE * const Buffer, MODULE_DESCRIPTOR * const ModuleDescriptor)
{
	WORD i;
	BYTE c;
   DWORD k, n;

	//    
	if(sscanf((char *)ModuleDescriptor->SerialNumber, "%u%c%lu", &k, &c, &n) != 0x3) { LAST_ERROR_NUMBER(22); return FALSE; }
	//  
	ZeroMemory(Buffer, 128);
	//   
	Buffer[0] = (BYTE)k;
	Buffer[1] = c;
	*(DWORD*)(Buffer+2) = n;
	//  
	sprintf((char *)(Buffer+6), "%.10s", ModuleDescriptor->Name);
	//  
	Buffer[16] = (BYTE)(ModuleDescriptor->Revision + (((ModuleDescriptor->Revision >= 'a') && (ModuleDescriptor->Revision <= 'z')) ? ('A'-'a') : 0));
	//   
	sprintf((char *)(Buffer+17), "%.10s", ModuleDescriptor->CpuType);
	//   
	*(DWORD*)(Buffer+27) = ModuleDescriptor->QuartzFrequency;
	//   
	Buffer[32] = (BYTE)(ModuleDescriptor->IsDacPresented ? 0x1 : 0x0);
	//    
	for(i = 0x0; i < ADC_CALIBR_COEFS_QUANTITY_E140; i++)
	{
		*(float*)(Buffer + 33 + (i + 0)*sizeof(float)) = (float)ModuleDescriptor->AdcOffsetCoefs[i];
		*(float*)(Buffer + 33 + (i + ADC_CALIBR_COEFS_QUANTITY_E140)*sizeof(float)) = (float)ModuleDescriptor->AdcScaleCoefs[i];
	}
	//    
	for(i = 0x0; i < DAC_CALIBR_COEFS_QUANTITY_E140; i++)
	{
		*(float*)(Buffer + 65 + (i + 0)*sizeof(float)) = (float)ModuleDescriptor->DacOffsetCoefs[i];
		*(float*)(Buffer + 65 + (i + DAC_CALIBR_COEFS_QUANTITY_E140)*sizeof(float)) = (float)ModuleDescriptor->DacScaleCoefs[i];
	}
	//     
	Buffer[31]  = CRC8CALC(Buffer+0, 31);
	//     
	Buffer[127] = CRC8CALC(Buffer+32, 95);
	//  
	return TRUE;
}

// -----------------------------------------------------------------------------
//    
// -----------------------------------------------------------------------------
BOOL TLE140::UnpackModuleDescriptor(BYTE * const Buffer, MODULE_DESCRIPTOR * const ModuleDescriptor)
{
	WORD i;

	//   
	if(CRC8CALC(Buffer+0, 31)  != Buffer[31]) { LAST_ERROR_NUMBER(24); return FALSE; }
	else if(CRC8CALC(Buffer+32, 95) != Buffer[127]) { LAST_ERROR_NUMBER(24); return FALSE; }
	//   
	sprintf((char *)(ModuleDescriptor->SerialNumber), "%u%c%lu", (DWORD)Buffer[0], Buffer[1], (*(DWORD*)(Buffer+2)) & 0xFFFFFFL);
	//  
	sprintf((char *)(ModuleDescriptor->Name), "%.10s", Buffer+6);
	//  
	ModuleDescriptor->Revision = Buffer[16];
	//   
	sprintf((char *)(ModuleDescriptor->CpuType), "%.10s", Buffer+17);
	//   
	ModuleDescriptor->QuartzFrequency = *(DWORD*)(Buffer+27);
	//      
	ModuleDescriptor->IsDacPresented = Buffer[32] ? true : false;
	//    
	for(i = 0x0; i < ADC_CALIBR_COEFS_QUANTITY_E140; i++)
	{
		ModuleDescriptor->AdcOffsetCoefs[i]	= *(float*)(Buffer + 33 + (i + 0)*sizeof(float));
		ModuleDescriptor->AdcScaleCoefs[i]	= *(float*)(Buffer + 33 + (i + ADC_CALIBR_COEFS_QUANTITY_E140)*sizeof(float));
	}
	//    
	for(i = 0x0; i < DAC_CALIBR_COEFS_QUANTITY_E140; i++)
	{
		ModuleDescriptor->DacOffsetCoefs[i]	= (*(float*)(Buffer + 65 + (i + 0)*sizeof(float)));
		ModuleDescriptor->DacScaleCoefs[i]	= (*(float*)(Buffer + 65 + (i + DAC_CALIBR_COEFS_QUANTITY_E140)*sizeof(float)));
	}
	//  
	return TRUE;
}

