// MainDlg.cpp : implementation of the CMainDlg class
//
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "resource.h"

#include "aboutdlg.h"
#include "DeviceFindDlg.h"
#include "RunADCDlg.h"
#include "MultimiterDlg.h"
#include "DacDlg.h"
#include "MainDlg.h"

BOOL CMainDlg::PreTranslateMessage(MSG* pMsg)
{
	return CWindow::IsDialogMessage(pMsg);
}

BOOL CMainDlg::OnIdle()
{
	return FALSE;
}

LRESULT CMainDlg::OnInitDialog(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	// center the dialog on the screen
	CenterWindow();

	// set icons
	HICON hIcon = (HICON)::LoadImage(_Module.GetResourceInstance(), MAKEINTRESOURCE(IDR_MAINFRAME), 
		IMAGE_ICON, ::GetSystemMetrics(SM_CXICON), ::GetSystemMetrics(SM_CYICON), LR_DEFAULTCOLOR);
	SetIcon(hIcon, TRUE);
	HICON hIconSmall = (HICON)::LoadImage(_Module.GetResourceInstance(), MAKEINTRESOURCE(IDR_MAINFRAME), 
		IMAGE_ICON, ::GetSystemMetrics(SM_CXSMICON), ::GetSystemMetrics(SM_CYSMICON), LR_DEFAULTCOLOR);
	SetIcon(hIconSmall, FALSE);	

	// Init full-screen dialog.
	SHINITDLGINFO shidi;
	ZeroMemory(&shidi, sizeof(shidi));
	shidi.dwMask = SHIDIM_FLAGS;
	shidi.dwFlags = SHIDIF_SIZEDLGFULLSCREEN;
	shidi.hDlg = m_hWnd;

	if (!SHInitDialog(&shidi))
	{
		ATLTRACE(_T("Failed to init dialog.\n"));
		::EndDialog(m_hWnd, 1);
	}

	// Create menu bar.
	SHMENUBARINFO mbi;
	memset(&mbi, 0, sizeof(mbi)); 
	mbi.cbSize = sizeof(mbi);
	mbi.hwndParent = m_hWnd;
	mbi.nToolBarId = IDR_MAIN_MENU;
	mbi.hInstRes = _Module.GetResourceInstance();

	if(!SHCreateMenuBar(&mbi))
	{
		ATLTRACE(_T("Failed to create menu bar in dialog.\n"));
		::EndDialog(m_hWnd, 2);
	}

	SCROLLINFO scinfo={sizeof(SCROLLINFO),SIF_ALL,0,0,0,0,0};
	this->GetScrollInfo(SB_VERT, &scinfo);
	scinfo.cbSize = sizeof(SCROLLINFO);
	scinfo.fMask=SIF_ALL;
	scinfo.nMax=500;
	scinfo.nMin=0;
	scinfo.nPos=0;	
	scinfo.nPage=10;
	this->SetScrollInfo(SB_VERT, &scinfo);	
	
	CComboBox ComboboxItem;	
	CButton CheckboxItem;
	for (int i=0;i<NumChannels;i++)
	{
		ComboboxItem=(CComboBox) this->GetDlgItem(ID_COMB_CHANNEL1+i);
		if (ComboboxItem!=NULL)
		{
			ComboboxItem.AddString(L"5000mV");
			ComboboxItem.AddString(L"1600mV");
			ComboboxItem.AddString(L"500mV");
			ComboboxItem.AddString(L"160mV");		
			ComboboxItem.SetCurSel(0);
		}	

		CheckboxItem = (CButton) this->GetDlgItem(ID_CHECK_CHANNEL1+i);
		if (CheckboxItem!=NULL)
		{
			CheckboxItem.SetCheck(1);
		}
	}	

	this->SetDlgItemText(ID_FREQ,L"120");
	this->SetDlgItemText(ID_ADC_TIME,L"60");
	this->SetDlgItemText(ID_ADC_WAIT_TIME,L"0");	

	memset(ConfFilePath,0,sizeof(ConfFilePath));	

	if (!GetModuleFileName((HMODULE)GetCurrentProcess(), ConfFilePath, sizeof(ConfFilePath)))
	{		
		AddLog("Unable Load Module Path");
	}	

	for (int i=wcslen(ConfFilePath);i>0;i--)
	{
		if (ConfFilePath[i]=='\\')
		{
			swprintf(&ConfFilePath[i],L"\\ExampleWTL.cnf");	
			break;
		}
	}		

	HANDLE hfile=INVALID_HANDLE_VALUE;
	if ((hfile=CreateFile(ConfFilePath, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING,0,NULL))!=INVALID_HANDLE_VALUE)
	{

		DWORD NumBytesReaded;
		if(ReadFile(hfile,&ExampleWTLConf, sizeof(ExampleWTLConf), &NumBytesReaded, NULL))
		{
			for (int i=0;i<NumChannels;i++)
			{
				ComboboxItem=(CComboBox) this->GetDlgItem(ID_COMB_CHANNEL1+i);
				if (ComboboxItem!=NULL)
				{			
					ComboboxItem.SetCurSel(ExampleWTLConf.ChannelRange[i]); 
				}	

				CheckboxItem = (CButton) this->GetDlgItem(ID_CHECK_CHANNEL1+i);
				if (CheckboxItem!=NULL)
				{
					CheckboxItem.SetCheck(ExampleWTLConf.ChannelEnabled[i]);
				}		
			}
			
			this->SetDlgItemInt(ID_FREQ, ExampleWTLConf.ADCFreqkHz,0);
			this->SetDlgItemInt(ID_ADC_TIME, ExampleWTLConf.TimeMeasureSec,0);
			this->SetDlgItemInt(ID_ADC_WAIT_TIME, ExampleWTLConf.KadrDelayMs,0);	 
		}
		CloseHandle(hfile);
	}

	// register object for message filtering and idle updates
	CMessageLoop* pLoop = _Module.GetMessageLoop();
	ATLASSERT(pLoop != NULL);
	pLoop->AddMessageFilter(this);
	pLoop->AddIdleHandler(this);

	UIAddChildWindowContainer(m_hWnd);	

	return TRUE;
}

LRESULT CMainDlg::OnAppAbout(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	CAboutDlg dlg;
	dlg.DoModal();
	return 0;
}

LRESULT CMainDlg::OnOK(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	// TODO: Add validation code 
	CloseDialog(wID);
	return 0;
}

LRESULT CMainDlg::OnCancel(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	CloseDialog(wID);
	return 0;
}

void CMainDlg::CloseDialog(int nVal)
{
	DestroyWindow();
	::PostQuitMessage(nVal);
}

LRESULT CMainDlg::OnHorzScroll(UINT uMsg, WPARAM wParam, LPARAM 
		lParam, BOOL& /*bHandled*/)
{
	
	
	return 0;
}

LRESULT CMainDlg::OnVertScroll(UINT uMsg, WPARAM wParam, LPARAM 
		lParam, BOOL& /*bHandled*/)
{	

	SCROLLINFO Info={sizeof(SCROLLINFO),SIF_ALL,0,0,0,0,0};
	this->GetScrollInfo(SB_VERT,&Info);	

	bool Arrows=false;

	switch (wParam&0xFFFF)
	{
	case 0:
		Info.nTrackPos=Info.nPos-5;
		Arrows=TRUE;
		break;
	case 1:
		Info.nTrackPos=Info.nPos+5;
		Arrows=TRUE;
		break;
	case 2:
		Info.nTrackPos=Info.nPos-Info.nPage;
		Arrows=TRUE;
		break;
	case 3:
		Info.nTrackPos=Info.nPos+Info.nPage;
		Arrows=TRUE;
		break;
	case 8: return 0;
	default: break;
	}
	
	Info.nTrackPos = min(Info.nTrackPos, Info.nMax);
	Info.nTrackPos = max(Info.nTrackPos, Info.nMin);		

	int Different = Info.nPos-Info.nTrackPos;
	if (Different&&lParam)
	{		
		this->ScrollWindowEx(0,Different,SW_INVALIDATE|SW_ERASE|SW_SCROLLCHILDREN);			
		this->SetScrollPos(SB_VERT, Info.nTrackPos);			
	}	

	return 0;
}


LRESULT CMainDlg::OnDAC_TTL(WORD wNotifyCode, WORD wID, 
		HWND hWndCtl, BOOL& bHandled)
{
	CDacDlg DltThis;

	if (DevicesDlg.CurrentSlot==-1) DevicesDlg.DoModal();

	DltThis.CurrentSlot=DevicesDlg.Slots[DevicesDlg.CurrentSlot];
	BOOL Trans=FALSE;	
	DltThis.ADCFreqkHz = this->GetDlgItemInt(ID_FREQ, &Trans,0);
	DltThis.KadrDelayMs=this->GetDlgItemInt(ID_ADC_WAIT_TIME, &Trans,0);
	
	
	CComboBox ComboboxItem;	
	CButton CheckboxItem;
	for (int i=0;i<NumChannels;i++)
	{
		ComboboxItem=(CComboBox) this->GetDlgItem(ID_COMB_CHANNEL1+i);
		if (ComboboxItem!=NULL)
		{			
			DltThis.ChannelRange[i]=ComboboxItem.GetCurSel();
		}	

		CheckboxItem = (CButton) this->GetDlgItem(ID_CHECK_CHANNEL1+i);
		if (CheckboxItem!=NULL)
		{
			DltThis.Channels[i]=CheckboxItem.GetCheck();
		}		
	}


	DltThis.DoModal();
	return 0;

	return TRUE;
}

LRESULT CMainDlg::OnMultimiter(WORD wNotifyCode, WORD wID, 
		HWND hWndCtl, BOOL& bHandled)
{

	CMultimiterDlg DltThis;

	if (DevicesDlg.CurrentSlot==-1) DevicesDlg.DoModal();

	DltThis.CurrentSlot=DevicesDlg.Slots[DevicesDlg.CurrentSlot];
	BOOL Trans=FALSE;	
	DltThis.ADCFreqkHz = this->GetDlgItemInt(ID_FREQ, &Trans,0);
	DltThis.KadrDelayMs=this->GetDlgItemInt(ID_ADC_WAIT_TIME, &Trans,0);
	
	
	CComboBox ComboboxItem;	
	CButton CheckboxItem;
	for (int i=0;i<NumChannels;i++)
	{
		ComboboxItem=(CComboBox) this->GetDlgItem(ID_COMB_CHANNEL1+i);
		if (ComboboxItem!=NULL)
		{			
			DltThis.ChannelRange[i]=ComboboxItem.GetCurSel();
		}	

		CheckboxItem = (CButton) this->GetDlgItem(ID_CHECK_CHANNEL1+i);
		if (CheckboxItem!=NULL)
		{
			DltThis.Channels[i]=CheckboxItem.GetCheck();
		}		
	}


	DltThis.DoModal();
	return 0;

	return TRUE;
}

LRESULT CMainDlg::OnRunADC(WORD wNotifyCode, WORD wID, 
		HWND hWndCtl, BOOL& bHandled)
{
	CRunADCDlg DltThis;

	if (DevicesDlg.CurrentSlot==-1) DevicesDlg.DoModal();

	DltThis.CurrentSlot=DevicesDlg.Slots[DevicesDlg.CurrentSlot];
	BOOL Trans=FALSE;	
	DltThis.ADCFreqkHz = this->GetDlgItemInt(ID_FREQ, &Trans,0);
	DltThis.GetTimeSec = this->GetDlgItemInt(ID_ADC_TIME, &Trans,0);
	DltThis.KadrDelayMs=this->GetDlgItemInt(ID_ADC_WAIT_TIME, &Trans,0);
	
	
	CComboBox ComboboxItem;	
	CButton CheckboxItem;
	for (int i=0;i<NumChannels;i++)
	{
		ComboboxItem=(CComboBox) this->GetDlgItem(ID_COMB_CHANNEL1+i);
		if (ComboboxItem!=NULL)
		{			
			DltThis.ChannelRange[i]=ComboboxItem.GetCurSel();
		}	

		CheckboxItem = (CButton) this->GetDlgItem(ID_CHECK_CHANNEL1+i);
		if (CheckboxItem!=NULL)
		{
			DltThis.Channels[i]=CheckboxItem.GetCheck();
		}		
	}


	DltThis.DoModal();
	return 0;
}

LRESULT CMainDlg::OnRunFindDevice(WORD wNotifyCode, WORD wID, 
		HWND hWndCtl, BOOL& bHandled)
{	
	DevicesDlg.DoModal();	
	return 0;
}

LRESULT CMainDlg::OnCheckChanged(WORD wNotifyCode, WORD wID, 
		HWND hWndCtl, BOOL& bHandled)
{	
	CButton CheckboxItem=(CButton)GetDlgItem(wID);
	if (CheckboxItem)
	{
		int ItemId = wID-ID_CHECK_CHANNEL1;
		ItemId+=ID_COMB_CHANNEL1;

		CComboBox ComboboxItem = (CComboBox)GetDlgItem(ItemId);
		if (ComboboxItem)
		{
			ComboboxItem.EnableWindow(CheckboxItem.GetCheck());
		}
	}

	return 0;
}

LRESULT CMainDlg::OnCloseDialog(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
{

	CComboBox ComboboxItem;	
	CButton CheckboxItem;
	
	for (int i=0;i<NumChannels;i++)
	{
		ComboboxItem=(CComboBox) this->GetDlgItem(ID_COMB_CHANNEL1+i);
		if (ComboboxItem!=NULL)
		{			
			ExampleWTLConf.ChannelRange[i]=ComboboxItem.GetCurSel();
		}	

		CheckboxItem = (CButton) this->GetDlgItem(ID_CHECK_CHANNEL1+i);
		if (CheckboxItem!=NULL)
		{
			ExampleWTLConf.ChannelEnabled[i]=CheckboxItem.GetCheck();
		}		
	}

	BOOL Trans = FALSE;
	ExampleWTLConf.ADCFreqkHz = this->GetDlgItemInt(ID_FREQ, &Trans,0);
	ExampleWTLConf.TimeMeasureSec = this->GetDlgItemInt(ID_ADC_TIME, &Trans,0);
	ExampleWTLConf.KadrDelayMs=this->GetDlgItemInt(ID_ADC_WAIT_TIME, &Trans,0);	 

	HANDLE hfile=INVALID_HANDLE_VALUE;
	if ((hfile=CreateFile(ConfFilePath, GENERIC_WRITE, FILE_SHARE_WRITE, NULL, OPEN_ALWAYS,0,NULL))!=INVALID_HANDLE_VALUE)
	{

		DWORD NumBytesWrited;
		if(!WriteFile(hfile,&ExampleWTLConf, sizeof(ExampleWTLConf), &NumBytesWrited, NULL))
		{
			AddLog("Unable Save config");
		}
		CloseHandle(hfile);
	}
	
    return 0;
}

