#include "stdafx.h"
#include "resource.h"

#include "MultimiterDlg.h"

CMultimiterDlg::CMultimiterDlg()
{	
	CurrentSlot=-1;
	Running=FALSE;
	CounterRead=0;
	pModule=NULL;
	ReadFromDeviceThread=INVALID_HANDLE_VALUE;
	ShowThread = INVALID_HANDLE_VALUE;
}

void CMultimiterDlg::AddPrintLog(CHAR * Message)
{
	AddLog (Message);
	TCHAR TempMessage[256];
	memset(TempMessage,0, sizeof(TempMessage));
	for (DWORD i=0;i<strlen(Message);i++)
	{
		TempMessage[i]=Message[i];
	}
	CListBox List = (CListBox) this->GetDlgItem(ID_MULT_LIST);
	
	if (List!=NULL)
	{
		List.AddString(TempMessage);	
		List.SetCurSel(List.GetCount()-1);
	}
}


void CMultimiterDlg::AbortProgramm(CHAR * Message)
{	
	Running = FALSE;
	AddPrintLog(Message);
	this->EndDialog(4);
}

LRESULT CMultimiterDlg::OnInitDialog(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	CenterWindow(GetParent());

	// Init full-screen dialog.
	SHINITDLGINFO shidi;
	ZeroMemory(&shidi, sizeof(shidi));
	shidi.dwMask = SHIDIM_FLAGS;
	shidi.dwFlags = SHIDIF_SIZEDLGFULLSCREEN;
	shidi.hDlg = m_hWnd;

	if (!SHInitDialog(&shidi))
	{
		ATLTRACE(_T("Failed to init dialog.\n"));
		this->EndDialog(1);
	}

	// Create menu bar.
	SHMENUBARINFO mbi;
	memset(&mbi, 0, sizeof(mbi)); 
	mbi.cbSize = sizeof(mbi);
	mbi.hwndParent = m_hWnd;
	mbi.nToolBarId = IDR_MULTIMITER_MENU;
	mbi.hInstRes = _Module.GetResourceInstance();	

	if(!SHCreateMenuBar(&mbi))
	{
		ATLTRACE(_T("Failed to create menu bar in dialog.\n"));
		this->EndDialog(2);
	}		


	for (int i=0;i<NumChannels;i++)
	{
		if (!Channels[i])
		{
			this->SetDlgItemText(ID_CH_VAL1+i, L"");
		}
	}

	return TRUE;
}

LRESULT CMultimiterDlg::OnCloseCmd(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{	
	EndDialog(wID);
	return 0;
}

void ReadThreadMult(PVOID Args)
{
	CMultimiterDlg * TmpDlg = (CMultimiterDlg*) Args;

	if(!TmpDlg->pModule->STOP_ADC()) TmpDlg->AbortProgramm("Error Stop ADC");

	IO_REQUEST_LUSBAPI IoReq;	
	
	WORD NumCh=0;
	for (int i=0;i<NumChannels;i++)
	{
		if (TmpDlg->Channels[i])NumCh++;
	}		

	DWORD NumWords = (((int)(TmpDlg->ADCFreqkHz*(1000/40)))*NumCh);		

	if(TmpDlg->KadrDelayMs)
	{
		NumWords = NumCh;
	}

	//       32
	NumWords = max(NumWords,32);		
	NumWords/=32;
	NumWords*=32; 
	
	double * Volts= new double[NumWords];
	if (Volts==NULL) TmpDlg->AbortProgramm("Error alloc memory");

	SHORT * Buf = new SHORT[NumWords];
	if (Buf==NULL) TmpDlg->AbortProgramm("Error alloc memory");	

	IoReq.Buffer = Buf;		
	IoReq.NumberOfWordsToPass = NumWords;
	IoReq.NumberOfWordsPassed = 0x0;
	IoReq.Overlapped = NULL;	
	IoReq.TimeOut = (DWORD)(1500);		

	if(TmpDlg->pModule->START_ADC())
	{		
		while(TmpDlg->Running)
		{			
			if(!TmpDlg->pModule->ReadData(&IoReq))
			{
				TmpDlg->AbortProgramm("Error recieve ADC Data");
				break;
			}

			TmpDlg->pModule->ProcessArray(IoReq.Buffer,Volts, NumWords, TRUE,TRUE);		
			
			for (int i=0, cnt=0;i<NumChannels;i++)
			{				
				if (TmpDlg->Channels[i])
				{				
					TmpDlg->chValues[i] = Volts[cnt++];					
				}				
			}	
			TmpDlg->DataReady = TRUE;
		}
	}else TmpDlg->AbortProgramm("Error Start ADC");

	if(!TmpDlg->pModule->STOP_ADC()) TmpDlg->AbortProgramm("Error Stop ADC");
	
	TmpDlg->AddPrintLog("Read Thread Complited");	

	TmpDlg->pModule->CloseLDevice();

	if (Volts!=NULL) delete Volts;
	if(Buf!=NULL) delete Buf;	
	
	ExitThread(1);
}

void ShowThreadMult(PVOID Args)
{
	CMultimiterDlg * TmpDlg = (CMultimiterDlg*) Args;	
		
	TCHAR Value[16];
	while(TmpDlg->Running)
	{		
		Sleep(40);

		if(!TmpDlg->DataReady) continue;

		for (int i=0, cnt=0;i<NumChannels;i++)
		{
			if (TmpDlg->Channels[i])
			{				
				memset(Value,0,sizeof(Value));
				wsprintf(Value,L"%4.8f", TmpDlg->chValues[i]);
				TmpDlg->SetDlgItemText(ID_CH_VAL1+i, Value);
			}			
		}	

		TmpDlg->CounterRead++;
		TmpDlg->SetDlgItemInt(ID_MULT_STAT, (DWORD)TmpDlg->CounterRead);					
	}	

	TmpDlg->AddPrintLog("Show Thread Complited");	

	ExitThread(1);
}

LRESULT CMultimiterDlg::OnDoneCmd(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{

	if (CurrentSlot==-1)
	{
		MessageBox(L" ");
		AbortProgramm("No Device");
	}

	if(Running)
	{
		Running = FALSE;				
		return 0;
	}
	if(ReadFromDeviceThread!=INVALID_HANDLE_VALUE)
	{		
		if(WaitForSingleObject(ReadFromDeviceThread, 100)!=WAIT_TIMEOUT) return FALSE;	
		ReadFromDeviceThread=INVALID_HANDLE_VALUE;
		CloseHandle(ReadFromDeviceThread);			
	}

	if(ShowThread!=INVALID_HANDLE_VALUE)
	{
		if(WaitForSingleObject(ShowThread, 100)!=WAIT_TIMEOUT) return FALSE;	
		ShowThread=INVALID_HANDLE_VALUE;
		CloseHandle(ShowThread);			
	}

	Running = TRUE;
	DataReady = FALSE;

	AddPrintLog(" *******************************");
	AddPrintLog(" Module E-154                   ");
	AddPrintLog(" example for ADC Stream ");
	AddPrintLog(" *******************************");

	//     Lusbapi.dll
	DWORD DllVersion;
	if((DllVersion = GetDllVersionLCARD()) != CURRENT_VERSION_LUSBAPI)
	{
		char String[128];
		sprintf(String, " Lusbapi.dll Version Error!!!   Current: %1u.%1u. Required: %1u.%1u",
			DllVersion >> 0x10, DllVersion & 0xFFFF,
			CURRENT_VERSION_LUSBAPI >> 0x10, CURRENT_VERSION_LUSBAPI & 0xFFFF);

		AbortProgramm(String);
	}
	else AddPrintLog("Lusbapi.dll Version --> OK");

	//     
	pModule = static_cast<ILE154 *>(CreateLInstance("e154"));
	if(!pModule) AbortProgramm(" Module Interface --> Bad");
	else AddPrintLog(" Module Interface --> OK");

	pModule->CloseLDevice();
	if(!pModule->OpenLDevice(CurrentSlot)) 
	{
		AbortProgramm("Error Open");
		return FALSE;
	}
	else AddPrintLogMessage1(" OpenLDevice(%u) --> OK", CurrentSlot);

	//    
	HANDLE ModuleHandle = pModule->GetModuleHandle();
	if(ModuleHandle == INVALID_HANDLE_VALUE)
	{
		AbortProgramm(" GetModuleHandle() --> Bad");
		return FALSE;
	}
	else AddPrintLog(" GetModuleHandle() --> OK");	

	//       
	CHAR ModuleName[16];
	if(!pModule->GetModuleName(ModuleName))
	{
		AbortProgramm(" GetModuleName() --> Bad");
		return FALSE;
	}
	else AddPrintLog(" GetModuleName() --> OK");
	// ,   'E-154'
	if(strcmp(ModuleName, "E154"))
	{
		AbortProgramm(" The module is not 'E-154'");
		return FALSE;
	}
	else AddPrintLog(" The module is 'E-154'");

	//    USB
	BYTE UsbSpeed;

	//      USB	
	if(!pModule->GetUsbSpeed(&UsbSpeed))
	{
		AbortProgramm(" GetUsbSpeed() --> Bad");
		return FALSE;
	}
	else AddPrintLog(" GetUsbSpeed() --> OK");
	//      USB
	AddPrintLogMessage1("   USB is in %s", UsbSpeed ? "High-Speed Mode (480 Mbit/s)" : "Full-Speed Mode (12 Mbit/s)");

	//      
	MODULE_DESCRIPTION_E154 ModuleDescription;

	//     	
	if(!pModule->GET_MODULE_DESCRIPTION(&ModuleDescription)) 
	{
		AbortProgramm(" GET_MODULE_DESCRIPTION() --> Bad");
		return FALSE;
	}
	else AddPrintLog(" GET_MODULE_DESCRIPTION() --> OK");

	//     
	ADC_PARS_E154 ap;
	//     
	if(!pModule->GET_ADC_PARS(&ap))
	{
		AbortProgramm(" GET_ADC_PARS() --> Bad");
		return FALSE;
	}
	else AddPrintLog(" GET_ADC_PARS() --> OK");

	//     
	ap.ClkSource = INT_ADC_CLOCK_E154;							//   
	ap.EnableClkOutput = ADC_CLOCK_TRANS_DISABLED_E154; 	//     
	ap.InputMode = NO_SYNC_E154;									//    
	ap.ChannelsQuantity = 0; 					// -  
	for (int i=0;i<NumChannels;i++)
	{
		if (Channels[i])
		{				
			ap.ControlTable[ap.ChannelsQuantity] = (WORD)(ap.ChannelsQuantity | (ChannelRange[i] << 0x6));
			ap.ChannelsQuantity++;
		}
	}
	//    	
	ap.AdcRate = ADCFreqkHz;						//     
	ap.InterKadrDelay =KadrDelayMs;							//    	

	//        
	AddPrintLog(" ");
	AddPrintLogMessage1(" Module E-154 (S/N %s) is ready ... ", ModuleDescription.Module.SerialNumber);
	AddPrintLog("   Module Info:");
	AddPrintLogMessage1("     Module  Revision   is '%c'", ModuleDescription.Module.Revision);
	AddPrintLogMessage2("     MCU Driver Version is %s (%s)", ModuleDescription.Mcu.Version.Version, ModuleDescription.Mcu.Version.Date);
	AddPrintLog("   Adc parameters:");
	AddPrintLogMessage1("     ChannelsQuantity = %2d", ap.ChannelsQuantity);
	AddPrintLogMessage1("     AdcRate = %8.3f kHz", ap.AdcRate);
	AddPrintLogMessage1("     InterKadrDelay = %2.4f ms", ap.InterKadrDelay);
	AddPrintLogMessage1("     KadrRate = %8.3f kHz", ap.KadrRate);

	this->SetDlgItemText(ID_TEXTBOX_TIME, L"0");	
	this->SetDlgItemText(ID_TEXTBOX_STATUS, L"0");

	//       
	if(!pModule->SET_ADC_PARS(&ap)) 
	{
		AbortProgramm(" SET_ADC_PARS() --> Bad");
		return FALSE;
	}
	else AddPrintLog(" SET_ADC_PARS() --> OK");	

	KillTimer(1);
	SetTimer(1,500);
	//      
	AddPrintLog(" "); 
	ReadFromDeviceThread = CreateThread(0, 0,(LPTHREAD_START_ROUTINE)ReadThreadMult, this, 0, 0);
	if(!ReadFromDeviceThread)
	{
		AbortProgramm(" ReadFromDeviceThread() --> Bad");
		return FALSE;
	}
	else AddPrintLog(" ReadFromDeviceThread() --> OK");

	ShowThread = CreateThread(0, 0,(LPTHREAD_START_ROUTINE)ShowThreadMult, this, 0, 0);
	if(!ShowThread)
	{
		AbortProgramm(" ShowThread() --> Bad");
		return FALSE;
	}
	else AddPrintLog(" ShowDeviceThread() --> OK");

	return 0;
}

LRESULT CMultimiterDlg::OnCloseDialog(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	Running = FALSE;
	if(ReadFromDeviceThread!=INVALID_HANDLE_VALUE)
	{
		if(WaitForSingleObject(ReadFromDeviceThread, 300)!=WAIT_OBJECT_0)
		{
			CloseHandle(ReadFromDeviceThread);	
		}		
	}

	if(ShowThread!=INVALID_HANDLE_VALUE)
	{
		if(WaitForSingleObject(ShowThread, 300)==WAIT_OBJECT_0)
		{
			CloseHandle(ShowThread);			
		}		
	}

	if(pModule!=NULL) pModule->CloseLDevice();

	return TRUE;
}



