#include "stdafx.h"
#include "resource.h"

#include "RunADCDlg.h"

CRunADCDlg::CRunADCDlg()
{	
	CurrentSlot=-1;
	Running=FALSE;
	CounterRead=0;
	CounterWrite=0;
	pModule = NULL;
	ReadFromDeviceThread = INVALID_HANDLE_VALUE;
	WriteToFileThread = INVALID_HANDLE_VALUE;
}

void CRunADCDlg::AddPrintLog(CHAR * Message)
{
	AddLog (Message);
	TCHAR TempMessage[256];
	memset(TempMessage,0, sizeof(TempMessage));
	for (DWORD i=0;i<strlen(Message);i++)
	{
		TempMessage[i]=Message[i];
	}
	CListBox List = (CListBox) this->GetDlgItem(ID_LISTBOX);
	
	if (List!=NULL)
	{
		List.AddString(TempMessage);	
		List.SetCurSel(List.GetCount()-1);
	}
}


void CRunADCDlg::AbortProgramm(CHAR * Message)
{	
	Running = FALSE;
	AddPrintLog(Message);
	this->EndDialog(4);	
}

LRESULT CRunADCDlg::OnInitDialog(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	CenterWindow(GetParent());

	// Init full-screen dialog.
	SHINITDLGINFO shidi;
	ZeroMemory(&shidi, sizeof(shidi));
	shidi.dwMask = SHIDIM_FLAGS;
	shidi.dwFlags = SHIDIF_SIZEDLGFULLSCREEN;
	shidi.hDlg = m_hWnd;

	if (!SHInitDialog(&shidi))
	{
		ATLTRACE(_T("Failed to init dialog.\n"));
		this->EndDialog(1);
	}

	// Create menu bar.
	SHMENUBARINFO mbi;
	memset(&mbi, 0, sizeof(mbi)); 
	mbi.cbSize = sizeof(mbi);
	mbi.hwndParent = m_hWnd;
	mbi.nToolBarId = IDR_ADC_MENU;
	mbi.hInstRes = _Module.GetResourceInstance();	

	if(!SHCreateMenuBar(&mbi))
	{
		ATLTRACE(_T("Failed to create menu bar in dialog.\n"));
		this->EndDialog(2);
	}		

	DataReadyEvent = CreateEvent(NULL,FALSE,FALSE,NULL);

	BufferRead[0] = new SHORT[NUM_READ_BUFF*(READ_BUFF_SIZE/2)];
	if(BufferRead[0]==NULL||DataReadyEvent==NULL)
	{
		AddLog("Error Alloc Buffer Memory");
		this->EndDialog(3);
	}

	for (int i=1;i<NUM_READ_BUFF;i++)
	{
		BufferRead[i] = (PSHORT)(((BYTE *)BufferRead[0])+(READ_BUFF_SIZE*i));
	}		

	return TRUE;
}

LRESULT CRunADCDlg::OnCloseCmd(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{

	EndDialog(wID);
	return 0;
}

void ReadThread(PVOID Args)
{	
	CRunADCDlg * TmpDlg = (CRunADCDlg*) Args;
	int BufferIndex=0;

	if(!TmpDlg->pModule->STOP_ADC()) TmpDlg->AbortProgramm("Error Stop ADC");

	IO_REQUEST_LUSBAPI * IoReq = new IO_REQUEST_LUSBAPI[NUM_READ_BUFF];
	if(IoReq==NULL)
	{
		TmpDlg->AbortProgramm("Error alloc request memory");
	}

	for(int i=0;i<NUM_READ_BUFF;i++)
	{
		IoReq[i].Buffer = (SHORT *)TmpDlg->BufferRead[i];
		IoReq[i].NumberOfWordsToPass = READ_BUFF_SIZE/2;
		IoReq[i].NumberOfWordsPassed = 0x0;
		IoReq[i].Overlapped = NULL;
		IoReq[i].TimeOut = (DWORD)((double)READ_BUFF_SIZE/TmpDlg->ADCFreqkHz + 1000);
	}

	if(TmpDlg->pModule->START_ADC())
	{		
		while(TmpDlg->Running&&(GetTickCount()<TmpDlg->EndTickCount))
		{
			if ((TmpDlg->CounterRead-TmpDlg->CounterWrite)>=NUM_READ_BUFF)
			{				
				AddLogMessage2("Counters Read %d, Write %d", (int)TmpDlg->CounterRead, (int)TmpDlg->CounterWrite);
				TmpDlg->AbortProgramm("Error counter Read overflow");
				break;
			}

			if(!TmpDlg->pModule->ReadData(&IoReq[BufferIndex]))
			{
				TmpDlg->AbortProgramm("Error recieve ADC Data");		
				break;				
			}

			TmpDlg->CounterRead++;
			BufferIndex=(BufferIndex+1)%NUM_READ_BUFF;
			SetEvent(TmpDlg->DataReadyEvent);
		}
	}else TmpDlg->AbortProgramm("Error Start ADC");

	if(!TmpDlg->pModule->STOP_ADC()) TmpDlg->AbortProgramm("Error Stop ADC2");

	TmpDlg->AddPrintLog("Read Thread Complited");

	if(IoReq!=NULL) delete IoReq;

	ExitThread(1);
}

void WriteThread(PVOID Args)
{
	CRunADCDlg * TmpDlg = (CRunADCDlg*) Args;

	int BufferIndex=0;	

	//        
	HANDLE hFile = CreateFile(L"\\SD Card\\Test.dat", GENERIC_READ|GENERIC_WRITE, FILE_SHARE_READ|FILE_SHARE_WRITE, 
		NULL, CREATE_ALWAYS, 0, NULL);
	if(hFile == INVALID_HANDLE_VALUE) TmpDlg->AbortProgramm("Can't create file 'Test.dat'!");

	DWORD BeginTick = GetTickCount();
	DWORD TenMinitesTick =BeginTick;
	DWORD SecondsTick =BeginTick;
	DWORD CurrentTickDiff = 0;

	TCHAR TemMes[128];

	while(TmpDlg->Running&&(GetTickCount()<TmpDlg->EndTickCount))
	{
		if(WaitForSingleObject(TmpDlg->DataReadyEvent, 500)==WAIT_OBJECT_0)
		{
			while (TmpDlg->CounterWrite<TmpDlg->CounterRead)
			{
				if ((TmpDlg->CounterRead-TmpDlg->CounterWrite)>NUM_READ_BUFF)
				{
					TmpDlg->AddPrintLog("Error counter Write overflow");
					AddLogMessage2("Counters Read %d, Write %d", (int)TmpDlg->CounterRead, (int)TmpDlg->CounterWrite);
					TmpDlg->CounterWrite=TmpDlg->CounterRead-NUM_READ_BUFF;
				}

				int NumBuffToWrite =(int)( TmpDlg->CounterRead-TmpDlg->CounterWrite);
				NumBuffToWrite = min(NumBuffToWrite, NUM_READ_BUFF-BufferIndex);

				//      
				DWORD FileBytesWritten=0;
				if(!WriteFile(	hFile,													// handle to file to write to
								TmpDlg->BufferRead[BufferIndex],					// pointer to data to write to file
								NumBuffToWrite*READ_BUFF_SIZE,	 											// number of bytes to write
	    						&FileBytesWritten,									// pointer to number of bytes written
						   		NULL			  											// pointer to structure needed for overlapped I/O
								)
							) 
				{
					TmpDlg->AbortProgramm("Error File Write 1");
					break;
				}

				if(FileBytesWritten!=(NumBuffToWrite*READ_BUFF_SIZE))
				{
					TmpDlg->AbortProgramm("Error File Write Size 1");
					break;
				}

				BufferIndex=(BufferIndex+NumBuffToWrite)%NUM_READ_BUFF;
				TmpDlg->CounterWrite+=NumBuffToWrite;
				

				CurrentTickDiff = GetTickCount() - BeginTick;
				
				if((CurrentTickDiff-SecondsTick)>1000)
				{					
					memset(TemMes,0,sizeof(TemMes));
					int Secs =TmpDlg->GetTimeSec-(CurrentTickDiff/1000);
					swprintf(TemMes,L" %d ", Secs);
					TmpDlg->SetDlgItemText(ID_TEXTBOX_TIME, TemMes);
					memset(TemMes,0,sizeof(TemMes));
					swprintf(TemMes,L"%d | %d", (int)TmpDlg->CounterRead, (int)TmpDlg->CounterWrite);
					TmpDlg->SetDlgItemText(ID_TEXTBOX_STATUS, TemMes);

					AddLogMessage2("Counters Read %d, Write %d", (int)TmpDlg->CounterRead, (int)TmpDlg->CounterWrite);	

					SecondsTick = CurrentTickDiff;
				}

				if((CurrentTickDiff-TenMinitesTick)>(1000*60*10))
				{	
					AddLogMessage1("Minutes, Counter=%d", CurrentTickDiff/(1000*60));

					TenMinitesTick = CurrentTickDiff;
				}				
			}
		}		
	}

	if (hFile!=INVALID_HANDLE_VALUE) CloseHandle(hFile);

	TmpDlg->Running = FALSE;

	TmpDlg->AddPrintLog("Write Thread Complited");	

	ExitThread(1);
}



LRESULT CRunADCDlg::OnDoneCmd(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	if (CurrentSlot==-1)
	{
		MessageBox(L" ");
		AbortProgramm("No Device");
	}

	if(Running) return FALSE;


	Running = TRUE;

	AddPrintLog(" *******************************");
	AddPrintLog(" Module E-154                   ");
	AddPrintLog(" example for ADC Stream ");
	AddPrintLog(" *******************************");

	//     Lusbapi.dll
	DWORD DllVersion;
	if((DllVersion = GetDllVersionLCARD()) != CURRENT_VERSION_LUSBAPI)
	{
		char String[128];
		sprintf(String, " Lusbapi.dll Version Error!!!   Current: %1u.%1u. Required: %1u.%1u",
			DllVersion >> 0x10, DllVersion & 0xFFFF,
			CURRENT_VERSION_LUSBAPI >> 0x10, CURRENT_VERSION_LUSBAPI & 0xFFFF);

		AbortProgramm(String);
	}
	else AddPrintLog("Lusbapi.dll Version --> OK");

	//     
	pModule = static_cast<ILE154 *>(CreateLInstance("e154"));
	if(!pModule) AbortProgramm(" Module Interface --> Bad");
	else AddPrintLog(" Module Interface --> OK");

	pModule->CloseLDevice();
	if(!pModule->OpenLDevice(CurrentSlot)) 
	{
		AbortProgramm("Error Open");
		return FALSE;
	}
	else AddPrintLogMessage1(" OpenLDevice(%u) --> OK", CurrentSlot);

	//    
	HANDLE ModuleHandle = pModule->GetModuleHandle();
	if(ModuleHandle == INVALID_HANDLE_VALUE)
	{
		AbortProgramm(" GetModuleHandle() --> Bad");
		return FALSE;
	}
	else AddPrintLog(" GetModuleHandle() --> OK");	

	//       
	CHAR ModuleName[16];
	if(!pModule->GetModuleName(ModuleName))
	{
		AbortProgramm(" GetModuleName() --> Bad");
		return FALSE;
	}
	else AddPrintLog(" GetModuleName() --> OK");
	// ,   'E-154'
	if(strcmp(ModuleName, "E154"))
	{
		AbortProgramm(" The module is not 'E-154'");
		return FALSE;
	}
	else AddPrintLog(" The module is 'E-154'");

	//    USB
	BYTE UsbSpeed;

	//      USB	
	if(!pModule->GetUsbSpeed(&UsbSpeed))
	{
		AbortProgramm(" GetUsbSpeed() --> Bad");
		return FALSE;
	}
	else AddPrintLog(" GetUsbSpeed() --> OK");
	//      USB
	AddPrintLogMessage1("   USB is in %s", UsbSpeed ? "High-Speed Mode (480 Mbit/s)" : "Full-Speed Mode (12 Mbit/s)");

	//      
	MODULE_DESCRIPTION_E154 ModuleDescription;

	//     	
	if(!pModule->GET_MODULE_DESCRIPTION(&ModuleDescription)) 
	{
		AbortProgramm(" GET_MODULE_DESCRIPTION() --> Bad");
		return FALSE;
	}
	else AddPrintLog(" GET_MODULE_DESCRIPTION() --> OK");

	//     
	ADC_PARS_E154 ap;
	//     
	if(!pModule->GET_ADC_PARS(&ap))
	{
		AbortProgramm(" GET_ADC_PARS() --> Bad");
		return FALSE;
	}
	else AddPrintLog(" GET_ADC_PARS() --> OK");

	//     
	ap.ClkSource = INT_ADC_CLOCK_E154;							//   
	ap.EnableClkOutput = ADC_CLOCK_TRANS_DISABLED_E154; 	//     
	ap.InputMode = NO_SYNC_E154;									//    
	ap.ChannelsQuantity = 0; 					// -  
	for (int i=0;i<NumChannels;i++)
	{
		if (Channels[i])
		{				
			ap.ControlTable[ap.ChannelsQuantity] = (WORD)(ap.ChannelsQuantity | (ChannelRange[i] << 0x6));
			ap.ChannelsQuantity++;
		}
	}
	//    	
	ap.AdcRate = ADCFreqkHz;						//     
	ap.InterKadrDelay = KadrDelayMs;							//    	

	//        
	AddPrintLog(" ");
	AddPrintLogMessage1(" Module E-154 (S/N %s) is ready ... ", ModuleDescription.Module.SerialNumber);
	AddPrintLog("   Module Info:");
	AddPrintLogMessage1("     Module  Revision   is '%c'", ModuleDescription.Module.Revision);
	AddPrintLogMessage2("     MCU Driver Version is %s (%s)", ModuleDescription.Mcu.Version.Version, ModuleDescription.Mcu.Version.Date);
	AddPrintLog("   Adc parameters:");
	AddPrintLogMessage1("     ChannelsQuantity = %2d", ap.ChannelsQuantity);
	AddPrintLogMessage1("     AdcRate = %8.3f kHz", ap.AdcRate);
	AddPrintLogMessage1("     InterKadrDelay = %2.4f ms", ap.InterKadrDelay);
	AddPrintLogMessage1("     KadrRate = %8.3f kHz", ap.KadrRate);

	this->SetDlgItemText(ID_TEXTBOX_TIME, L"0");	
	this->SetDlgItemText(ID_TEXTBOX_STATUS, L"0");

	//       
	if(!pModule->SET_ADC_PARS(&ap)) 
	{
		AbortProgramm(" SET_ADC_PARS() --> Bad");
		return FALSE;
	}
	else AddPrintLog(" SET_ADC_PARS() --> OK");	
	
	//      
	AddPrintLog(" "); 
	EndTickCount = GetTickCount()+(GetTimeSec*1000);	

	WriteToFileThread = CreateThread(0, 0, (LPTHREAD_START_ROUTINE)WriteThread, this, 0, 0);
	SetThreadPriority(WriteToFileThread, THREAD_PRIORITY_ABOVE_NORMAL);
	if(!WriteToFileThread)
	{
		AbortProgramm(" WriteToFileThread() --> Bad");
		return FALSE;
	}
	else AddPrintLog(" WriteToFileThread() --> OK");

	ReadFromDeviceThread = CreateThread(0, 0,(LPTHREAD_START_ROUTINE)ReadThread, this, 0, 0);
	SetThreadPriority(ReadFromDeviceThread, THREAD_PRIORITY_ABOVE_NORMAL);
	if(!ReadFromDeviceThread)
	{
		AbortProgramm(" ReadFromDeviceThread() --> Bad");
		return FALSE;
	}
	else AddPrintLog(" ReadFromDeviceThread() --> OK");


	return 0;
}



LRESULT CRunADCDlg::OnCloseDialog(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	Running = FALSE;
	if(ReadFromDeviceThread!=INVALID_HANDLE_VALUE)
	{
		if(WaitForSingleObject(ReadFromDeviceThread, 300)==WAIT_OBJECT_0)	
		{
			CloseHandle(ReadFromDeviceThread);
		}
	}

	if(WriteToFileThread!=INVALID_HANDLE_VALUE)
	{
		if(WaitForSingleObject(WriteToFileThread, 300)==WAIT_OBJECT_0)	
		{
			CloseHandle(WriteToFileThread);
		}		
	}

	if (DataReadyEvent!=NULL) CloseHandle(DataReadyEvent);
	
	if(BufferRead[0]!=NULL) delete BufferRead[0];
	
	if (pModule!=NULL)pModule->CloseLDevice();

	return TRUE;
}



